package com.cometway.jerminal;

import java.io.*;
import java.util.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


public class ClassLoader extends java.lang.ClassLoader
{
	Hashtable cache;
	Hashtable dataCache;

	public ClassLoader()
	{
		cache = new Hashtable();
		dataCache = new Hashtable();
	}

	public synchronized Class loadClass(String name, boolean resolve)
	{
		Class c = findLoadedClass(name);

		if (c == null)
			c = (Class) cache.get(name);

		if (c == null)
		{
			byte[] data = loadClassData(name);

			if (data != null)
			{
				c = defineClass(null, data, 0, data.length);
				cache.put(name, c);
			}
		}

		if (c == null)
		{
			try
			{
				c = findSystemClass(name);
			}
			catch (Exception e)
			{
				System.out.println("pb2q.lang.ClassLoader excepted trying to findSystemClass");
			}
		}

		if ((c != null) && resolve)
			resolveClass(c);
	
		return c;
	}

	/* this method is the reason for this class.  This should be provided by
	 *  java.lang.ClassLoader but isn't
	 */
	public synchronized Class loadClass(byte[] classData)
	{
		Class c = defineClass(null, classData, 0, classData.length);

		return c;
	}

	public synchronized Class loadClass(String name, byte[] classData)
	{
		Class c = null;

		if (cache.containsKey(name))
		{
			c = (Class) cache.get(name);
		}
 		else
		{
			c = defineClass(name, classData, 0, classData.length);
		
			if (c != null)
				cache.put(name, c);
		}

		if (!dataCache.containsKey(name))
			dataCache.put(name, classData);

		return c;
	}

	/* class data can be retrieved based upon a loaded class using this method,
	 *  if the Class was loaded using this ClassLoader or obtained from an
	 *  instance of such a Class.
	 */
	public byte[] getClassData(Class clas)
	{
		String name = clas.getName();

		if (dataCache.containsKey(name))
			return (byte[]) dataCache.get(name);
		else return null;
	}

	/* implements ClassDataProvider.getClassData(String) and provides access to
	 *  class data given a String identifier (probably a fully qualified Class
	 *  name).
	 */
	public byte[] getClassData(String identifier)
	{
		byte[] data = null;
		
		if (dataCache.containsKey(identifier))
			data = (byte[]) dataCache.get(identifier);
		else
		{
			String dirs = System.getProperty("java.class.path");
			java.util.StringTokenizer strtok =
				new java.util.StringTokenizer(dirs, ":"); 
			String dir = "";

			while ((data == null) && strtok.hasMoreTokens())
			{
				dir = strtok.nextToken();
				data = checkForClassFile(dir, identifier);
			}
		}

		return data;
	}

	private static byte[] checkForClassFile(String dir, String identifier)
	{
		byte[] data = null;

		java.util.StringTokenizer strtok =
			new java.util.StringTokenizer(identifier, ".");
		String fileString = "";

		while (strtok.hasMoreTokens())
			fileString += File.separatorChar + strtok.nextToken();

		fileString = dir + fileString + ".class";

		File classFile = new File(fileString);

		if (classFile.isFile())
			data = loadClassFile(classFile);

		return data;
	}

	public static byte[] loadClassFile(File classFile)
	{
		int BLOCK_SIZE = 512;

		boolean success    = false;
		FileInputStream in = null;
		byte[] classData   = null;

		try
		{
			in = new FileInputStream(classFile);

			int size    = 1024;
			int offset  = 0;
			int numRead = -1;
			classData   = new byte[size];

			while (in.available() > 0)
			{
				if (offset >= size)
				{
					byte[] newData = new byte[2 * size];
					System.arraycopy(classData, 0, newData, 0, size);
					size = 2 * size;
					classData = newData;
				}

				numRead = in.read(classData, offset, BLOCK_SIZE);

				if (numRead != -1)
					offset += numRead;
			}

			in.close();

			if (offset != size)
			{
				byte[] newData = new byte[offset];
				System.arraycopy(classData, 0, newData, 0, offset);
				classData = newData;
			}

			success = true;
		}
		catch (Exception e)
		{
			success = false;
		}

		return classData;
	}

	/* this method is intended to be overridden by subclasses which might have
	 *  special methods for obtaining class data.  for example a TCPClassLoader
	 *  might be instantiated with a hostname and port which it can use to grab
	 *  class data given a name.  This version of the method does nothing.
	 */
	protected byte[] loadClassData(String name)
	{
		return null;
	}
}
