
package com.cometway.jerminal;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import java.awt.*;
import java.awt.event.*;

import kinetoscope.util.*;

/**
 * This implementation of JerminalKeyListener interprets the keyboard input using the
 * Emacs keyset. Editting keys behave like emacs editing keys.
 */
public class EmacsKeyListener implements JerminalKeyListener
{
	/** This keylistener's parent JerminalWindow, keyboard input is read from here */
	public JerminalWindow parent;
	/** This is the input buffer. keystrokes are appended to this buffer until a newline is read */
	public StringBuffer buffer;
	/** This is the cursor index of the input buffer. This should be in constant offset to the JerminalWindow's cursor */
	public int index;

	/** This flag determines whether this key listener should interpret the input as Emacs keys */
	public boolean interpret;
	/** This flag determines whether input should be inserted at the cursor or overwritten */
	public boolean insert;

	/** This buffer is the copy/paste buffer */
	public String killBuffer;

	protected int historyIndex;
	protected String tmpHistory;

	protected boolean shift;
	protected boolean ctrl;
	protected boolean alt;

	/**
	 * This constructor assigns the parent to the JerminalWindow parameter. All fields are 
	 * initialized in this constructor.
	 */
	public EmacsKeyListener(JerminalWindow parent)
	{
		this.parent = parent;

		buffer = new StringBuffer();
		index = 0;
		interpret = true;
		insert = true;
		historyIndex=-1;
	}


	/**************** These methods implement the JerminalKeyListener interface **********/


	public void setInsert(boolean i)
	{
		insert = i;
	}

	public boolean getInsert()
	{
		return(insert);
	}

	public void setInterpret(boolean i)
	{
		interpret = i;
	}
	
	public boolean getInterpret()
	{
		return(interpret);
	}

	public void setKillBuffer(String buff)
	{
		killBuffer = buff;
	}

	public String getKillBuffer()
	{
		return(killBuffer);
	}

	public void setInputBuffer(String buff)
	{
		buffer = new StringBuffer(buff);
	}

	public String getInputBuffer()
	{
		return(buffer.toString());
	}

	public void setBufferIndex(int i)
	{
		if(i<0) {
			i=0;
		}
		if(i>buffer.length()) {
			i = buffer.length();
		}
		index = i;
	}

	public int getBufferIndex()
	{
		return(index);
	}

	public void setJerminalWindow(JerminalWindow window)
	{
		parent = window;
		parent.keyListener = this;
	}

	public JerminalWindow getJerminalWindow()
	{
		return(parent);
	}




	/************** These methods implement the KeyListener interface ******************/


	public void keyPressed(KeyEvent event)
	{
		//		parent.print("Got key pressed: "+event);
		if(interpret) {
			try {
				if(event.getKeyCode()==event.VK_SHIFT) {
					shift = true;
				}
				if(event.getKeyCode()==event.VK_CONTROL) {
					ctrl = true;
				}
				else if(event.getKeyCode()==event.VK_ALT) {
					alt = true;
				}
				else if(event.getKeyCode()==event.VK_META) {
					alt = true;
				}


				else if(event.getKeyCode()==event.VK_A) {
					if(ctrl && !alt) {
						beginningOfLine();
					}
					else if(shift && !ctrl && !alt) {
						event.setKeyChar('A');
						defaultKey(event);
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_B) {
					if(ctrl && !alt) {
						back();
					}
					else if(!ctrl && alt) {
						wordBack();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_C) {
					if(ctrl && !alt) {
						keyBreak();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_D) {
					if(ctrl && !alt) {
						delete();
					}
					else if(alt && !ctrl) {
						deleteWordForward();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_E) {
					if(ctrl && !alt) {
						endOfLine();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_F) {
					if(ctrl && !alt) {
						forward();
					}
					else if(!ctrl && alt) {
						wordForward();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_K) {
					if(ctrl && !alt) {
						kill();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_L) {
					if(ctrl && !alt) {
						parent.refresh(true);
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_N) {
					if(ctrl && !alt) {
						next();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_P) {
					if(ctrl && !alt) {
						previous();
					}
					else {
						defaultKey(event);
					}
				}
				else if(event.getKeyCode()==event.VK_Y) {
					if(ctrl && !alt) {
						yank();
					}
					else {
						defaultKey(event);
					}
				}


				else if(event.getKeyCode()==event.VK_NUMPAD0) {
					event.setKeyChar('0');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD1) {
					event.setKeyChar('1');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD2) {
					event.setKeyChar('2');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD3) {
					event.setKeyChar('3');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD4) {
					event.setKeyChar('4');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD5) {
					event.setKeyChar('5');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD6) {
					event.setKeyChar('6');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD7) {
					event.setKeyChar('7');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD8) {
					event.setKeyChar('8');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_NUMPAD9) {
					event.setKeyChar('9');
					defaultKey(event);
				}


				else if(event.getKeyCode()==event.VK_UP) {
					previous();
				}
				else if(event.getKeyCode()==event.VK_DOWN) {
					next();
				}
				else if(event.getKeyCode()==event.VK_LEFT) {
					if(ctrl && !alt) {
						wordBack();
					}
					else if(!ctrl && alt) {
						deleteWordBack();
					}
					else {
						back();
					}
				}
				else if(event.getKeyCode()==event.VK_RIGHT) {
					if(ctrl && !alt) {
						wordForward();
					}
					else if(!ctrl && alt) {
						deleteWordForward();
					}
					else {
						forward();
					}
				}

				/*
				else if(event.getKeyCode()==event.VK_KP_UP) {
					previous();
				}
				else if(event.getKeyCode()==event.VK_KP_DOWN) {
					next();
				}
				else if(event.getKeyCode()==event.VK_KP_LEFT) {
					if(ctrl && !alt) {
						wordBack();
					}
					else if(!ctrl && alt) {
						deleteWordBack();
					}
					else {
						back();
					}
				}
				else if(event.getKeyCode()==event.VK_KP_RIGHT) {
					if(ctrl && !alt) {
						wordForward();
					}
					else if(!ctrl && alt) {
						deleteWordForward();
					}
					else {
						forward();
					}
				}
				*/

				else if(event.getKeyCode()==event.VK_INSERT) {
					insert = !insert;
				}
				else if(event.getKeyCode()==event.VK_END) {
					endOfLine();
				}
				else if(event.getKeyCode()==event.VK_ENTER) {
					newline();
				}
				else if(event.getKeyCode()==event.VK_HOME) {
					beginningOfLine();
				}
				else if(event.getKeyCode()==event.VK_DELETE) {
					if(alt && !ctrl) {
						deleteWordBack();
					}
					else {
						backspace();
					}
				}
				else if(event.getKeyCode()==event.VK_BACK_SPACE) {
					if(alt && !ctrl) {
						deleteWordBack();
					}
					else {
						backspace();
					}
				}
				
				
				else if(event.getKeyCode()==event.VK_TAB) {
					complete();
				}
				else if(event.getKeyCode()==event.VK_ESCAPE) {
					event.setKeyChar('\\');
					defaultKey(event);
				}
				else if(event.getKeyCode()==event.VK_PAGE_UP) {
					if(shift && !ctrl && !alt) {
						parent.scrollUp();
					}
				}
				else if(event.getKeyCode()==event.VK_PAGE_DOWN) {
					if(shift && !ctrl && !alt) {
						parent.scrollDown();
					}
				}
				else {
					if(event.getKeyCode()!=16 && event.getKeyCode()!=17 && event.getKeyCode()!=18) {
						defaultKey(event);
					}
				}
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void keyReleased(KeyEvent event)
	{
		//		parent.print("Got key released: "+event);
		if(interpret) {
			try {
				if(event.getKeyCode()==event.VK_SHIFT) {
					shift = false;
				}
				else if(event.getKeyCode()==event.VK_CONTROL) {
					ctrl = false;
				}
				else if(event.getKeyCode()==event.VK_ALT) {
					alt = false;
				}
				else if(event.getKeyCode()==event.VK_META) {
					alt = false;
				}
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void keyTyped(KeyEvent event)
	{
		//		parent.print("Got key types: "+event);
		if(!interpret) {
			defaultKey(event);
		}
			
	}





	/**
	 * This method handles command completion and file/path completion. The list of commands
	 * are given as a property in the JerminalWindow called 'commands_directory', and in this 
	 * directory, this method will attempt to complete on commands. Anything that is not the first
	 * word in the input buffer is assumed to be a file/path. 
	 */
	protected void complete()
	{
		try {
			String tmp = buffer.toString();
			String completeString = null;
			boolean completeCommand = false;
			if(index == 0) {
				completeCommand = true;
				completeString = tmp.substring(0,index);
			}
			else {
				int i1 = index;
				//				if(i1==tmp.length()) {
				//					i1--;
				//				}
				int i2 = tmp.substring(0,i1).lastIndexOf(" ");
				if(i2==-1) {
					if(i1!=0) {
						try {
							completeString = tmp.substring(0,index);
							completeCommand = true;
						}
						catch(Exception e) {;}
					}
					else {
						completeString = "";
						completeCommand = true;
					}
				}
				else {
					try {
						completeString = tmp.substring(i2,index);
					}
					catch(Exception e) {;}
				}
			}
			completeString = completeString.trim();
			//			System.out.println("COMPLETE STRING: "+completeString+", command="+completeCommand);

			Vector completionList = new Vector();

			if(completeCommand) {
				try {
					//					Vector v = Jerminal.commaToVector(parent.props.getString("commands_directory"));
					// currently, can't do this with commands in another package
					File commandsDir = new File(parent.props.getString("commands_directory"));
					String[] commands = commandsDir.list();
					for(int x=0;x<commands.length;x++) {
						try {
							if(commands[x].indexOf(".class")!=-1) {
								commands[x] = commands[x].substring(0,commands[x].indexOf(".class"));
								Class com = Class.forName("com.cometway.jerminal.commands."+commands[x]);
								//								System.out.println("COMMAND? "+commands[x]);
								if(!Modifier.isAbstract(com.getModifiers()) && 
									!Modifier.isInterface(com.getModifiers())) {
									if(completeString==null || completeString.length()==0) {
										completionList.addElement(new Pair(commands[x],null));
									}
									else {
										if(commands[x].startsWith(completeString)) {
											completionList.addElement(new Pair(commands[x],null));
										}
									}
								}
							}
						}
						catch(NoClassDefFoundError fu) {;}
						catch(Exception e) {;}
					}
				}
				catch(Exception e) {
					//					e.printStackTrace();
					parent.visualBeep();
					return;
				}
			}
			else {
				// This must be a file completion
				char pathSep = parent.jerminal.getProps().getString("path_separator_char").charAt(0);
				String[] files = (new File(parent.jerminal.getProps().getString("pwd"))).list();

				if(completeString.startsWith("~")) {
					String home = parent.jerminal.getProps().getString("home_directory");
					if(home.charAt(home.length()-1)==pathSep) {
						home = home.substring(0,home.length()-1);
					}
					completeString = home+completeString.substring(1);
				}
				
				if(completeString==null || completeString.length()==0) {
					try {
						for(int x=0;x<files.length;x++) {
							File f = new File(files[x]);
							if(f.isDirectory()) {
								completionList.addElement(new Pair(files[x]+pathSep,pathSep+""));
							}
								else {
									completionList.addElement(new Pair(files[x],null));
								}
						}
					}
					catch(Exception e) {
						e.printStackTrace();
					}
				}
				else if((new File(parent.jerminal.getProps().getString("pwd"),completeString)).exists()) {
					try {
						File f = new File(parent.jerminal.getProps().getString("pwd"),completeString);
						if(f.isDirectory()) {
							files = f.list();
							for(int x=0;x<files.length;x++) {
								if((new File(f,files[x])).isDirectory()) {
									if(completeString.charAt(completeString.length()-1)==pathSep) {
										completionList.addElement(new Pair(completeString+files[x]+pathSep,pathSep+""));
									}
									else {
										completionList.addElement(new Pair(completeString+pathSep+files[x]+pathSep,pathSep+""));
									}
								}
								else {
									if(completeString.charAt(completeString.length()-1)==pathSep) {
										completionList.addElement(new Pair(completeString+files[x],null));
									}
									else {
										completionList.addElement(new Pair(completeString+pathSep+files[x],null));
									}
								}
							}
						}
						else {
							parent.visualBeep();
							return;
						}
					}
					catch(Exception e) {
						e.printStackTrace();
					}
				}
				else {
					int lastSep = completeString.lastIndexOf(pathSep);
					if(lastSep==-1) {
						for(int x=0;x<files.length;x++) {
							if(files[x].startsWith(completeString)) {
								File f = new File(parent.jerminal.getProps().getString("pwd"),files[x]);
								if(f.isDirectory()) {
									completionList.addElement(new Pair(files[x]+pathSep,pathSep+""));
								}
								else {
									completionList.addElement(new Pair(files[x],null));
								}
							}
						}
					}
					else {
						try {
							if(completeString.startsWith(pathSep+"") || 
								(completeString.charAt(1)==':' && completeString.charAt(2)==pathSep)) {
								File test = null;
								if(completeString.substring(0,lastSep).length()==0) {
									test = new File(completeString.substring(0,lastSep+1));
								}
								else {
									test = new File(completeString.substring(0,lastSep));
								}
								if(test.exists()) {
									String tmp2 = completeString.substring(lastSep+1);
									files = test.list();
									for(int x=0;x<files.length;x++) {
										if(files[x].startsWith(tmp2)) {
											File f = new File(test,files[x]);
											if(f.isDirectory()) {
												completionList.addElement(new Pair(completeString.substring(0,lastSep+1)+files[x]+pathSep,pathSep+""));
											}
											else {
												completionList.addElement(new Pair(completeString.substring(0,lastSep+1)+files[x],null));
											}
										}
									}
								}
								else {
									parent.visualBeep();
									return;
								}
							}
							else {
								File test = new File(parent.jerminal.getProps().getString("pwd"),
															completeString.substring(0,lastSep));
								if(test.exists()) {
									String tmp2 = completeString.substring(lastSep+1);
									files = test.list();
									for(int x=0;x<files.length;x++) {
										if(files[x].startsWith(tmp2)) {
											File f = new File(test,files[x]);
											if(f.isDirectory()) {
												completionList.addElement(new Pair(completeString.substring(0,lastSep+1)+files[x]+pathSep,pathSep+""));
											}
											else {
												completionList.addElement(new Pair(completeString.substring(0,lastSep+1)+files[x],null));
											}
										}
									}
								}
								else {
									parent.visualBeep();
									return;
								}
							}
						}
						catch(Exception e) {
							e.printStackTrace();
						}
					}
				}
			}

			//System.out.println(completionList);
			if(completionList.size()>1) {
				//				completionList = StringTools.pairCaseSensitiveSort(completionList);
				String add = "";
				for(int x=0;x<1000;x++) {
					try {
						Pair p = (Pair)completionList.elementAt(0);
						char tmpChar = ((String)p.first()).charAt(x);
						for(int z=1;z<completionList.size();z++) {
							p = (Pair)completionList.elementAt(z);
							if(tmpChar != ((String)p.first()).charAt(x)) {
								x=1000;
								break;
							}
						}
						if(x!=1000) {
							add = add+tmpChar;
						}
					}
					catch(Exception e) {
						break;
					}
				}

				//				else {
				int cursorPos = parent.cursorX;
				String parentString = parent.text[parent.cursorY];
				parent.makeNewLine();
				int longest = 0;
				for(int x=0;x<completionList.size();x++) {
					Pair p = (Pair)completionList.elementAt(x);
					if(((String)p.first()).length()>longest) {
						longest = ((String)p.first()).length();
					}
				}

				int i = (int)(parent.getMaxCharWidth(parent.fixedFontWidth)/(longest+1));
				//				if(i==0) {i=1;}
				parent.moveCursor(0,parent.cursorY);
				int savedWidth = parent.tabWidth;
				parent.tabWidth=longest+2;
				for(int x=0;x<completionList.size();x++) {
					if(i==0 || x%i==0) {
						parent.makeNewLine();
						parent.moveCursor(0,parent.cursorY);
					}
					Pair p = (Pair)completionList.elementAt(x);
					parent.insertAtCursor((String)p.first());
					parent.moveCursorX(((String)p.first()).length());
					if(i!=0) {
						parent.insertTab();
					}
				}
				if(completeString.length()>0) {
					try {
						//							int y = completeString.indexOf(add.charAt(0));
						//							if(y==0) {
						add = add.substring(completeString.length());
						//							}
						//							else {
						//								add = add.substring(completeString.length()-y);
						//							}
					}
					catch(Exception e) {
						add = "";
					}
				}
				parent.tabWidth = savedWidth;
				parent.makeNewLine();
				parent.moveCursor(0,parent.cursorY);
				buffer = new StringBuffer(tmp.substring(0,index)+add+tmp.substring(index));
				parent.text[parent.cursorY] = parentString.substring(0,cursorPos-index)+
					buffer.toString();

				parent.moveCursor(cursorPos+add.length(),parent.cursorY);
				index = index+add.length();
				//				}
				parent.refresh(true);
			}
			else if(completionList.size()==1) {
				Pair p = (Pair)completionList.elementAt(0);
				String add = ((String)p.first());
				//				int y = completeString.indexOf(add.charAt(0));
				//				if(y==0) {
					add = add.substring(completeString.length());
					//				}
					//				else {
					//					add = add.substring(completeString.length()-y);
					//				}

					//System.out.println("ADD= "+add);
					//System.out.println("COMPLETESTRING= "+completeString); 
					//System.out.println("pathSep? "+p.second());

				buffer = new StringBuffer(tmp.substring(0,index));
				buffer.append(add);
				if(p.second()==null) {
					buffer.append(" ");
					buffer.append(tmp.substring(index));
					index = index+ add.length()+1;
					parent.insertAtCursor(add+" ");
					parent.moveCursorX(1+add.length());
				}
				else {
					buffer.append(tmp.substring(index));
					index = index+ add.length();
					parent.insertAtCursor(add);
					parent.moveCursorX(add.length());
				}					
				parent.refresh(false);
			}
			else {
				parent.visualBeep();
			}
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener while trying to complete",e);
		}
	}


	/**
	 * This helper method simply displays the key inputed to the JerminalWindow.
	 */
	protected void defaultKey(KeyEvent event)
	{
		try {
			char c = event.getKeyChar();
			String old = parent.text[parent.cursorY];
			if(insert) {
				buffer.insert(index,c);
				index++;
				parent.insertAtCursor(c);
				parent.moveCursorX(1);
			}
			else {
				String tmp = buffer.toString();
				buffer = new StringBuffer(tmp.substring(0,index));
				buffer.append(c);
				buffer.append(tmp.substring(index+1));
				index++;
				parent.overwriteAtCursor(c);
				parent.moveCursorX(1);
			}
			parent.refresh(!lineDiff(old,parent.text[parent.cursorY]));
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method starts a newline in the JerminalWindow and flushes the input buffer
	 * to the JerminalWindow's JerminalSession (thus giving it a command). The indeces and cursor is reset
	 * to the beginning. 
	 */
	protected void newline()
	{
		try {
			parent.makeNewLine();
			
			parent.jerminalOut.write(buffer.toString().getBytes());
			parent.jerminalOut.write(10);
			parent.jerminalOut.flush();
		
			//			if(buffer.toString().equals("exit")) {
			//				System.exit(0);
			//			}
			index = 0;
			historyIndex = -1;
			tmpHistory = "";
			buffer = new StringBuffer();
			parent.refresh(true);
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method tells the JerminalWindow to delete a character backwards. The same is 
	 * done in the input buffer.
	 */
	protected void backspace()
	{
		try {
			if(index==0) {
				parent.visualBeep();
			}
			else {
				String old = parent.text[parent.cursorY];
				if(buffer.length()>0) {
					String tmp = buffer.toString();
					try {
						buffer = new StringBuffer(tmp.substring(0,index-1));
						buffer.append(tmp.substring(index));
					}
					catch(Exception e) {
						//					parent.visualBeep();
					}
					parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX-1) + 
						parent.text[parent.cursorY].substring(parent.cursorX);
					
					parent.moveCursorX(-1);
					index--;
				}
				parent.refresh(!lineDiff(old,parent.text[parent.cursorY]));
			}
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method moves the cursor and the input buffer index to the beginning.
	 */	
	protected void beginningOfLine()
	{
		parent.moveCursorX(-1*index);
		index=0;
		parent.refresh(false);
	}

	/**
	 * This helper method moves the cursor and the input buffer index to the end.
	 */
	protected void endOfLine()
	{
		parent.moveCursorX(buffer.length()-index);
		index=buffer.length();
		parent.refresh(false);
	}

	/**
	 * This helper method moves the cursor and the input buffer back one character.
	 */
	protected void back()
	{
		index--;
		if(index<0) {
			parent.visualBeep();
			index=0;
		}
		else {
			parent.moveCursorX(-1);
		}
		parent.refresh(false);
	}

	/**
	 * This helper method moves the cursor and the input buffer forward one character.
	 */
	protected void forward()
	{
		if(index<buffer.length()) {
			index++;
			parent.moveCursorX(1);
		}
		else {
			parent.visualBeep();
		}
		parent.refresh(false);
	}

	/**
	 * This helper method displays the previous command in the command history to the
	 * JerminalWindow and sets the cursor and the input buffer index to the end.
	 */
	protected void previous()
	{
		try {
			if(historyIndex==-1) {
				tmpHistory = buffer.toString();
			}
			historyIndex++;
			if(parent.jerminal.commandHistory.size()<=historyIndex) {
				historyIndex = -1;
			}
			if(historyIndex<-1) {
				historyIndex=-1;
			}
			parent.moveCursorX(-1*index);
			parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX);
			if(historyIndex==-1) {
				buffer = new StringBuffer(tmpHistory);
						}
			else {
				buffer = new StringBuffer((String)parent.jerminal.commandHistory.elementAt(historyIndex));
			}
			index = buffer.length();
			parent.text[parent.cursorY] = parent.text[parent.cursorY]+buffer;
			parent.moveCursorX(index);
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
		parent.refresh(false);
	}
	
	/**
	 * This helper method displays the next command in the command history to the
	 * JerminalWindow and sets the cursor and the input buffer index to the end.
	 */
	protected void next()
	{
		try {
			if(historyIndex==-1) {
				tmpHistory = buffer.toString();
			}
			historyIndex--;
			if(parent.jerminal.commandHistory.size()<=historyIndex) {
				historyIndex=parent.jerminal.commandHistory.size()-1;
			}
			if(historyIndex<-1) {
				historyIndex=parent.jerminal.commandHistory.size()-1;
			}
			parent.moveCursorX(-1*index);
			parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX);
			if(historyIndex==-1) {
				buffer = new StringBuffer(tmpHistory);
			}
			else {
				buffer = new StringBuffer((String)parent.jerminal.commandHistory.elementAt(historyIndex));
			}
			index = buffer.length();
			parent.text[parent.cursorY] = parent.text[parent.cursorY]+buffer;
			parent.moveCursorX(index);
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
		parent.refresh(false);
	}

	/**
	 * This helper method displays a '^C' on a line by itself to the JerminalWindow.
	 * Whatever was in the input buffer is cleared and nothing is sent to the JerminalSession.
	 */
	protected void keyBreak()
	{
		try {
			parent.makeNewLine();
			
			parent.text[parent.cursorY] = new String("^C");
			
			parent.makeNewLine();
			//			parent.visualBeep();
			
			index = 0;
			buffer = new StringBuffer();
			historyIndex = -1;
			tmpHistory = "";
			
			parent.jerminalOut.write((int)10);
			parent.jerminalOut.flush();
			parent.refresh(true);
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method deletes one character at the input buffer index.
	 */
	protected void delete()
	{
		try {
			String old = parent.text[parent.cursorY];
			String tmp = buffer.toString();
			buffer = new StringBuffer(tmp.substring(0,index));
			try {
				buffer.append(tmp.substring(index+1));
				tmp = parent.text[parent.cursorY].substring(parent.cursorX+1);
			}
			catch(Exception e) {
				parent.visualBeep();
				tmp="";
			}
			parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX)+tmp;
			parent.refresh(!lineDiff(old,parent.text[parent.cursorY]));
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method deletes all the characters from the input buffer index to the end of the
	 * input buffer and stores that String in the killBuffer. The JerminalWindow is also updated.
	 */
	protected void kill()
	{
		try {
			String old = parent.text[parent.cursorY];
			
			buffer = new StringBuffer(buffer.toString().substring(0,index));
			killBuffer = parent.text[parent.cursorY].substring(parent.cursorX);				
			parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX);
			
			parent.refresh(!lineDiff(old,parent.text[parent.cursorY]));
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method inserts the killBuffer at the input buffer index and updates the
	 * JerminalWindow.
	 */
	protected void yank()
	{
		try {
			if(killBuffer!=null) {
				String old = parent.text[parent.cursorY];
				
				if(insert) {
					buffer.insert(index,killBuffer);
					index = index+killBuffer.length();
					parent.insertAtCursor(killBuffer);
					parent.moveCursorX(killBuffer.length());
				}
				else {
					String tmp = buffer.toString();
					buffer = new StringBuffer(tmp.substring(0,index));
					buffer.append(killBuffer);
					buffer.append(tmp.substring(index+killBuffer.length()));
					parent.overwriteAtCursor(killBuffer);
					parent.moveCursorX(killBuffer.length());
				}					
				parent.refresh(!lineDiff(old,parent.text[parent.cursorY]));
			}
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method deletes up to the next whitespace or path separator or end of line from the 
	 * input buffer index forward. The JerminalWindow is also updated.
	 */
	protected void deleteWordForward()
	{
		try {
			String tmp = buffer.toString();
			int i2 = index;
			while(i2<tmp.length() && tmp.charAt(i2)==' ') {
				i2++;
			}
			if(i2==tmp.length()) {
				i2--;
			}
			char pathSep = parent.jerminal.getProps().getString("path_separator_char").charAt(0);
			int i1 = tmp.indexOf(" ",i2);
			int i3 = tmp.indexOf(pathSep,i2); 
			if(i1!=-1 && i3!=-1) {
				if(i1>i3) {
					i1 = i3+1;
				}
			}
			else {
				if(i3!=-1) {
					i1 = i3+1;
				}
			}

			if(i1==-1) {
				try {
					if(tmp.substring(index).length()>0) {
						killBuffer = tmp.substring(index);
					}
					else {
						parent.visualBeep();
					}
				}
				catch(Exception e) {
					parent.visualBeep();
				}
				buffer = new StringBuffer(tmp.substring(0,index));
				parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX-index)+buffer;
			}
			else {
				try {
					killBuffer = tmp.substring(index,i1);
				}
				catch(Exception e) {
					parent.visualBeep();
				}
				buffer = new StringBuffer(tmp.substring(0,index));
				buffer.append(tmp.substring(i1));
				parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX-index)+buffer;
			}
			parent.refresh(true);
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method deletes up to the previous whitespace or path separator or beginning of line from the 
	 * input buffer index backwards. The JerminalWindow is also updated.
	 */
	protected void deleteWordBack()
	{
		try {
			String tmp = buffer.toString();
			int i1 = index;
			char pathSep = parent.jerminal.getProps().getString("path_separator_char").charAt(0);
			if(i1==tmp.length()) {
				i1--;
			}
			if(i1!=0 && (tmp.charAt(i1-1)==' ') || tmp.charAt(i1-1)==pathSep) {
				while(i1>0 && (tmp.charAt(i1-1)==' ' || tmp.charAt(i1-1)==pathSep)) {
					i1--;
				}
			}
			int i2 = tmp.substring(0,i1).lastIndexOf(" ");
			int i3 = tmp.substring(0,i1).lastIndexOf(pathSep); 
			if(i2!=-1 && i3!=-1) {
				if(i2 < i3) {
					i2 = i3;
				}
			}
			else {
				if(i3!=-1) {
					i2 = i3;
				}
			}

			if(i2==-1) {
				if(i1!=0) {
					try {
						if(tmp.substring(index).length()>0) {
							killBuffer = tmp.substring(0,index);
						}
						else {
							parent.visualBeep();
						}
					}
					catch(Exception e) {
						parent.visualBeep();
					}
					buffer = new StringBuffer(tmp.substring(index));
					parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX-index)+buffer;
					parent.moveCursorX(-1*index);
					index = 0;
				}
			}
			else {
				try {
					killBuffer = tmp.substring(i2,index);
				}
				catch(Exception e) {
					parent.visualBeep();
				}
				buffer = new StringBuffer(tmp.substring(0,i2));
				buffer.append(tmp.substring(index));
				parent.text[parent.cursorY] = parent.text[parent.cursorY].substring(0,parent.cursorX-index)+buffer;
				parent.moveCursorX(-1*(index-i2));
				index = i2;
			}
			parent.refresh(true);
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
	}

	/**
	 * This helper method moves the cursor and the input buffer index to the next whitespace or path separator
	 * from the input buffer index forward. The JerminalWindow is updated.
	 */
	protected void wordForward()
	{
		try {
			String tmp = buffer.toString();
			int i2 = index;
			while(i2<tmp.length() && tmp.charAt(i2)==' ') {
				i2++;
			}
			if(i2==tmp.length()) {
				i2--;
			}
			char pathSep = parent.jerminal.getProps().getString("path_separator_char").charAt(0);
			int i1 = tmp.indexOf(" ",i2);
			int i3 = tmp.indexOf(pathSep,i2); 
			if(i1!=-1 && i3!=-1) {
				if(i1>i3) {
					i1 = i3+1;
				}
			}
			else {
				if(i3!=-1) {
					i1 = i3+1;
				}
			}

			if(i1==-1) {
				parent.moveCursorX(buffer.length()-index);
				index = buffer.length();
			}
			else {
				parent.moveCursorX(i1-index);
				index = i1;
			}
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
		parent.refresh(false);
	}

	/**
	 * This helper method moves the cursor and the input buffer index to the previous whitespace or path separator 
	 * from the input buffer index backwards. The JerminalWindow is updated.
	 */
	protected void wordBack()
	{
		try {
			String tmp = buffer.toString();
			int i1 = index;
			if(i1==tmp.length()) {
				i1--;
			}
			if(i1!=0 && tmp.charAt(i1-1)==' ') {
				while(i1>0 && tmp.charAt(i1-1)==' ') {
					i1--;
				}
			}
			char pathSep = parent.jerminal.getProps().getString("path_separator_char").charAt(0);
			int i2 = tmp.substring(0,i1).lastIndexOf(" ");
			int i3 = tmp.substring(0,i1).lastIndexOf(pathSep); 
			if(i2!=-1 && i3!=-1) {
				if(i2 < i3) {
					i2 = i3-1;
				}
			}
			else {
				if(i3!=-1) {
					i2 = i3-1;
				}
			}

			if(i2==-1) {
				parent.moveCursorX(-1*index);
				index = 0;
			}
			else {
				parent.moveCursorX(-1*(index-i2-1));
				index = i2+1;
			}
		}
		catch(Exception e) {
			parent.error("Error in keyboard listener",e);
		}
		parent.refresh(false);
	}



	/**
	 * This helper method helps determine how many lines the Strings passed in take up in the
	 * JerminalWindow. This method returns true if the 2 strings take up the same number of lines.
	 */
	protected boolean lineDiff(String in1, String in2)
	{
		int maxWidth = parent.getMaxCharWidth(parent.fixedFontWidth);
		int i1 = (int)(in1.length()/maxWidth);
		int i2 = (int)(in2.length()/maxWidth);
		if(in1.length()%maxWidth > 0) {
			i1++;
		}
		if(in2.length()%maxWidth > 0) {
			i2++;
		}
		//System.out.println("in1="+in1.length()+" in2="+in2.length());
		return(i1==i2);
	}

}
