package com.cometway.jerminal;

import java.util.*;
import java.io.*;
import java.net.*;

import kinetoscope.util.*;

import com.cometway.jak.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This Agent will listen to a port and instantiate a single JerminalSession on request. When a connection
 * is made to this agent, a password is requested. If the password is valid, this agent will
 * connect the client to the JerminalSession. The JerminalSession can be suspended if the 
 * client connection is closed, and can be resumed when the connection is reestablished.
 */
public class JerminalAgent extends Agent implements Runnable
{
	/** Setting this value to true will cause the JerminalAgent to stop listening on its ServerSocket. */
	public boolean stopRunning;

	JerminalSession jerminal;
	Thread runThread;
	
	public void initProps()
	{
		props.setDefault("listen_port","7870");
		props.setDefault("password","qazmko'68qwerty");
		
		props.setDefault("allow_file_access","true");
		props.setDefault("allow_binary_access","true");

		props.setDefault("default_environment","./environment.props");
	}


	public void start()
	{
		runThread = new Thread(this);
		runThread.setName("Jerminal Agent");

		runThread.start();
	}

	public void run()
	{
		try {
			boolean busy = false;
			ServerSocket ss = new ServerSocket(props.getInteger("listen_port"));
			while(!stopRunning) {
				Socket sock = ss.accept();

				BufferedReader reader = null;
				BufferedWriter writer = null;
				String password = null;
				try {
					reader = new BufferedReader(new InputStreamReader(sock.getInputStream()));
					writer = new BufferedWriter(new OutputStreamWriter(sock.getOutputStream()));
					
					writer.write("access code: ");
					writer.flush();
					
					password = reader.readLine();
				}
				catch(Exception e) {
					error("Error negotiating connection",e);
				}
				
				if(password.equals(props.getString("password"))) {
					if(jerminal == null) {
						jerminal = new JerminalSession(sock.getInputStream(),sock.getOutputStream(),Props.loadProps(props.getString("default_environment")),
														props.getBoolean("allow_file_access"),props.getBoolean("allow_binary_access"));
					}
					else {
						jerminal.in = sock.getInputStream();
						jerminal.out = sock.getOutputStream();
						jerminal.localPrintStream = new PrintStream(jerminal.out);
					}

					jerminal.run();
				}
				else {
					writer.write("Incorrect code\n\n");
					try {
						reader.close();
					}
					catch(Exception e) {;}
					try {
						writer.close();
					}
					catch(Exception e) {;}
					try {
							sock.close();
					}
					catch(Exception e) {;}
				}

				try {
						reader.close();
				}
				catch(Exception e) {;}
				try {
					writer.close();
				}
				catch(Exception e) {;}
				try {
					sock.close();
				}
				catch(Exception e) {;}
			}
		}
		catch(Exception e) {
			error("Error in run",e);
		}
	}
}
