

package com.cometway.jerminal;

import java.awt.event.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This interface defines the KeyListener which will read the keyboard input
 * from a JerminalWindow. The JerminalKeyistener ensures that the text is 
 * displayed correctly in the window and the commands typed are sent to the Jerminal
 * associated with the JerminalWindow.
 */
public interface JerminalKeyListener extends KeyListener
{
	/**
	 * This sets the INSERT flag of the key listener. If set to true, it is expected
	 * that keystrokes read from the JerminalWindow are INSERTed at the cursor.
	 */
	public void setInsert(boolean insert);

	/**
	 * This method returns the state of the INSERT flag.
	 */
	public boolean getInsert();

	/**
	 * This sets the INTERPRET flag of the key listener. If set to false, the key
	 * listener is expected not to interpret the input.
	 */
	public void setInterpret(boolean interpret);
	
	/**
	 * This method returns the state of the INTERPRET flag.
	 */
	public boolean getInterpret();

	/**
	 * This sets the copy/paste buffer of the key listener.
	 */
	public void setKillBuffer(String buffer);

	/**
	 * This method returns the copy/paste buffer.
	 */
	public String getKillBuffer();

	/**
	 * This method sets the input buffer of the key listener. This is the buffer that
	 * will be sent as a command to the jerminal.
	 */
	public void setInputBuffer(String buffer);

	/**
	 * This method returns what is currently in the input buffer.
	 */
	public String getInputBuffer();

	/**
	 * This method sets the cursor index of the input buffer.
	 */
	public void setBufferIndex(int index);

	/**
	 * This method returns the cursor index of the input buffer.
	 */
	public int getBufferIndex();

	/**
	 * This method sets the JerminalWindow which this key listener should be reading
	 * keyboard input from.
	 */
	public void setJerminalWindow(JerminalWindow window);

	/**
	 * This method returns the JerminalWindow which this key listener is reading from.
	 */
	public JerminalWindow getJerminalWindow();
}
