
package com.cometway.jerminal;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import kinetoscope.util.*;


/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This subclass of java.awt.Frame supports a Text terminal and many features that windowed
 * text terminals have. This window has an array of Strings which it paints in the window
 * starting from the bottom of the window and the end of the array. Lines that are longer than
 * the window width are automatically wrapped, thus reducing the total number of Strings
 * in the array which can be drawn. This window also keeps track of a cursor and provides a number
 * of methods which allow insertion of text at the cursor. There is also functionality to allow
 * scrolling through the rest of the Strings in the array which are not drawn in the window.
 */
public class JerminalWindow extends Frame implements ComponentListener, WindowListener
{
	/** The background color */
	public Color bgColor;
	/** The foreground color */
	public Color fgColor;

	/** The width of the window in pixels */
	public int width;
	/** The height of the window in pixels */
	public int height;

	/** The X offset (in pixels) of where the text is to be drawn */
	public int xoffset;
	/** The Y offset (in pixels) of where the text is to be drawn */
	public int yoffset;

	/** The number of pixels to buffer the left and right edge of the window */
	public int xCharOffset;
	/** The number of pixels to buffer the top and bottom edge of the window */
	public int yCharOffset;

	/** The font in which the text is drawn */
	public Font font;

	/** The X index of the cursor (index of the current String element in the array) */
	public int cursorX;
	/** The Y index of the cursor (index in the array of Strings) */
	public int cursorY;

	/** The number of pixels wide the current font takes up */
	public int fixedFontWidth;
	/** The number of pixels high the current font takes up */
	public int fixedFontHeight;




	/** The JerminalSession associated with this Window (this can be left null) */
	public JerminalSession jerminal;
	/** The JerminalSession input stream associated with this Window (this can be left null) */
	public InputStream jerminalIn;
	/** The JerminalSession output stream associated with this Window (this can be left null) */
	public OutputStream jerminalOut;

	/** This flag tells the JerminalWindow whether to draw a cursor */
	public boolean drawCursor;
	
	/** The key listener that will listen to keyboard input for this window */
	public JerminalKeyListener keyListener;

	/** The runtime properties for the JerminalWindow */
	public Props props;

	/** This flag tells the Jerminal window whether to exit the JerminalSession if a close window event is recieved. */
	public boolean exitOnClose = true;

	/** This is the width of a tab in characters */
	public int tabWidth;

	/** This flag is set to true when the JerminalWindow is currently scrolling through previously undrawn String array elements */
	public boolean scrolling;
	/** When scrolling, this is how far from the end of the array is being drawn */
	public int scrollIndex;

	/** This is the array that the text in this window is stored in. The paint() method draws the text in this array starting from the end of the array */
	public String[] text;


	/**
	 * Initialize window and local variables.
	 */
	public JerminalWindow(Props p)
	{
		super();

		props = p;
		initProps();
		init();

		show();
		repaint();
		setTitle("Jerminal Window");
		setName("Jerminal Window");
		setCursor(Cursor.TEXT_CURSOR);
		repaint();
	}

	/**
	 * Initialize defaults for Props and set local props dependant variables
	 */
	public void initProps()
	{
		//		props.setDefault("font","Monospaced");
		props.setDefault("font","DialogInput");
		props.setDefault("font_size","9");
		props.setDefault("bg","000000000");
		props.setDefault("fg","255000000");

		props.setDefault("xoffset","6");
		props.setDefault("yoffset","23");

		props.setDefault("xCharOffset","3");
		props.setDefault("yCharOffset","4");
		
		props.setDefault("xFrame","10");
		props.setDefault("yFrame","30");

		props.setDefault("initial_width","80");
		props.setDefault("initial_height","45");

		props.setDefault("scroll_back_size","250");
		props.setDefault("scroll_lines","25");
		props.setDefault("tab_length","10");

		props.setDefault("commands_directory","../java/com/cometway/jerminal/commands");
	}

	/**
	 * Init runtime variables
	 */
	public void init()
	{
		//		Toolkit kit = Toolkit.getDefaultToolkit();
		//		String[] fonts = kit.getFontList();
		//		for(int x=0;x<fonts.length;x++) {
		//			print(fonts[x]);
		//		}

		//		font = Font.getFont(props.getString("font"));
		font = new Font(props.getString("font"),Font.PLAIN,props.getInteger("font_size"));
		setFont(font);

		String color = props.getString("bg");
		int rColor = Integer.parseInt(color.substring(0,3));
		int bColor = Integer.parseInt(color.substring(3,6));
		int gColor = Integer.parseInt(color.substring(6,9));

		bgColor = new Color(rColor,bColor,gColor);

		color = props.getString("fg");
		rColor = Integer.parseInt(color.substring(0,3));
		bColor = Integer.parseInt(color.substring(3,6));
		gColor = Integer.parseInt(color.substring(6,9));
		fgColor = new Color(rColor,bColor,gColor);
		//		bgColor = Color.black;
		//		fgColor = Color.white;

		xoffset = props.getInteger("xoffset");
		yoffset = props.getInteger("yoffset");
		xCharOffset = props.getInteger("xCharOffset");
		yCharOffset = props.getInteger("yCharOffset");
		fixedFontWidth = props.getInteger("fixed_font_width");
		fixedFontHeight = props.getInteger("fixed_font_height");
		tabWidth = props.getInteger("tab_length");

		text = new String[props.getInteger("scroll_back_size")];
		for(int x=0;x<text.length;x++) {
			text[x] = "";
				//				x+":01234567890-0123456789-0123456789-0123456789-0123456789-0123456789-01234567890";
		}

		setBackground(bgColor);
		show();

		scrolling = false;

		addComponentListener(this);
		addWindowListener(this);

		Graphics g = getGraphics();
		g.setFont(font);
		FontMetrics fm = g.getFontMetrics();
		if(fixedFontWidth==0) {
			fixedFontWidth= fm.getMaxAdvance();
		}
		width = fixedFontWidth*props.getInteger("initial_width") + xCharOffset*2 + props.getInteger("xFrame");
		if(fixedFontHeight==0) {
			fixedFontHeight=fm.getHeight();
		}
		height = fixedFontHeight*props.getInteger("initial_height") + yCharOffset*2 + props.getInteger("yFrame");
		setSize(width,height);

		cursorX = 0;
		cursorY = text.length-1;
		drawCursor = true;
	}




	/**
	 * This method inserts a tab in the String array at the cursor. The length of the tab stop is given by the tabWidth field.
	 */
	public void insertTab()
	{
		if(text[cursorY]==null) {
			text[cursorY] = new String();
		}
		String spaces = "";
		int num = tabWidth-(text[cursorY].length()%tabWidth);
		for(int x=0;x<num;x++) {
			spaces = spaces+" ";
		}

		text[cursorY] = text[cursorY].substring(0,cursorX)+spaces+text[cursorY].substring(cursorX);
		cursorX = cursorX+num;
		scrolling = false;
	}

	/**
	 * This method inserts the String into the String array at the cursor.
	 */
	public void insertAtCursor(String s)
	{
		if(text[cursorY]==null) {
			text[cursorY] = new String();
		}
		text[cursorY] = text[cursorY].substring(0,cursorX)+s+text[cursorY].substring(cursorX);
	}

	/**
	 * This method inserts the character into the String array at the cursor.
	 */
	public void insertAtCursor(char c)
	{
		if(text[cursorY]==null) {
			text[cursorY] = new String();
		}
		text[cursorY] = text[cursorY].substring(0,cursorX)+c+text[cursorY].substring(cursorX);
		scrolling = false;
	}

	/**
	 * This method replaces the text at the cursor with the String passed in as a parameter.
	 */
	public void overwriteAtCursor(String s)
	{
		if(text[cursorY]==null) {
			text[cursorY] = new String();
		}
		if(cursorX+s.length()>text[cursorY].length()) {
			text[cursorY] = text[cursorY].substring(0,cursorX)+s;
		}
		else {
			text[cursorY] = text[cursorY].substring(0,cursorX)+s+text[cursorY].substring(cursorX+s.length());
		}
	}

	/**
	 * This method replaces the character at the cursor with the character passed in as a parameter.
	 */
	public void overwriteAtCursor(char c)
	{
		if(text[cursorY]==null) {
			text[cursorY] = new String();
		}
		if(text[cursorY].length()==cursorX) {
			text[cursorY] = text[cursorY].substring(0,cursorX)+c;
		}
		else {
			text[cursorY] = text[cursorY].substring(0,cursorX)+c+text[cursorY].substring(cursorX+1);
		}

		scrolling = false;
	}

	/**
	 * This method moves the cursor to the given coordinates. If the Y cursor is out of the array
	 * bounds, it is moved back into the end/beginning of the array. Then if the X cursor is out of 
	 * the String's bounds, it is moved to the end/beginning of the String.
	 */
	public void moveCursor(int x, int y)
	{
		if(y>=text.length) {
			y = text.length-1;
		}
		try {
			if(x>=text[y].length()) {
				x = text[y].length();
			}
		}
		catch(Exception e) {
			x = 0;
		}
		cursorX = x;
		cursorY = y;
		scrolling = false;
	}

	/**
	 * This method offsets the X cursor xdiff number of characters. If it leaves the bounds of the
	 * String, it will be moved to the end/beginning of the String.
	 */
	public void moveCursorX(int xdiff)
	{
		int x = cursorX + xdiff;
		try {
			if(x>=text[cursorY].length()) {
				x = text[cursorY].length();
			}
			else if(x<0) {
				x = 0;
			}
		}
		catch(Exception e) {
			x = 0;
		}
		cursorX = x;
		scrolling = false;
	}

	/**
	 * This method offsets the Y cursor xdiff number of array elements. If it leaves the bounds of the
	 * String array, it will be moved to the end/beginning of the array.
	 */
	public void moveCursorY(int ydiff)
	{
		int y = cursorY + ydiff;
		if(y>=text.length) {
			y = text.length-1;
		}
		else if(y<0) {
			y = 0;
		}
		cursorY=y;
		scrolling = false;
	}

	/**
	 * This method inserts a new String (new line) at the Y cursor. All the previous Strings from the
	 * Y cursor on are moved one less array index. The String at array index 0 is discarded.
	 */
	public void makeNewLine()
	{
		if(cursorY == text.length-1) {
			for(int x=0;x<cursorY;x++) {
				text[x] = text[x+1];
			}
			text[cursorY] = new String();
		}
		else {
			cursorY++;
		}
		cursorX=0;
		scrolling = false;
	}

	/**
	 * This method causes the window to flash, indicating a 'beep'.
	 */
	public void visualBeep()
	{
		try {
			Color tmpColor = bgColor;
			bgColor = fgColor;
			fgColor = tmpColor;
			Graphics g = getGraphics();
			g.setColor(bgColor);
			g.setXORMode(fgColor);
			g.fillRect(0,0,width+25,height+25);
			Thread.sleep(100);
			tmpColor = bgColor;
			bgColor = fgColor;
			fgColor = tmpColor;
			g = getGraphics();
			g.setColor(bgColor);
			g.fillRect(0,0,width+25,height+25);
			refresh(true);
		}		
		catch(Exception e) {
			e.printStackTrace();
		}
	}







	/**
	 * This method returns the maximum number of characters that will fit horizontally across
	 * the current window based on the given font advance.
	 */
	public int getMaxCharWidth(int advance)
	{
		return((int)((width-((xCharOffset*2)+props.getInteger("xFrame")))/advance));
	}

	/**
	 * This method returns the maximum number of characters that will fit vertically across 
	 * the current window based on the given font height.
	 */
	public int getMaxCharHeight(int fontheight)
	{
		return((int)((height-((yCharOffset*2)+props.getInteger("yFrame")))/fontheight));
	}

	/**
	 * This method causes the window to scroll down, thus viewing Strings of a higher array index. If the
	 * String at the end of the array can already be viewed, nothing will happen. This only really
	 * does anything if you have scrolled up first.
	 */
	public void scrollDown()
	{
		if(scrolling) {
			scrollIndex = scrollIndex+props.getInteger("scroll_lines");
			//		if(scrollIndex>(text.length-getMaxCharHeight(getFontMetrics().charWidth(' ')))) {
			
			//			System.out.println("scroll_index="+scrollIndex);
			if(scrollIndex>=text.length) {
				scrollIndex = text.length;
				scrolling = false;
			}
			refresh(true);
		}
	}

	/**
	 * This method causes the window to scroll up, thus viewing Strings of a lower array index. 
	 * If the String at the beginning of the array can already be viewed, nothing will happend.
	 */
	public void scrollUp()
	{
		scrolling = true;
		scrollIndex = scrollIndex-props.getInteger("scroll_lines");
		if(scrollIndex<0) {
			scrollIndex = 0;
		}
		refresh(true);
	}








	/**********
	 * WindowListener methods
	 *
	 */


	public void windowActivated(WindowEvent event) 
	{
		;
	}

	public void windowClosed(WindowEvent event) 
	{
		;
	}

	public void windowClosing(WindowEvent event) 
	{
		hide();
		if(exitOnClose) {
			System.exit(1);
		}
	}

	public void windowDeactivated(WindowEvent event) 
	{
		;
	}

	public void windowDeiconified(WindowEvent event) 
	{
		;
	}

	public void windowIconified(WindowEvent event) 
	{
		;
	}

	public void windowOpened(WindowEvent event)
	{
		;
	}





	/****
	 * ComponentListener methods
	 *
	 **/
	public void componentHidden(ComponentEvent event)
	{
		;
	}

	public void componentMoved(ComponentEvent event)
	{
		;
	}

	public void componentShown(ComponentEvent event)
	{
		;
	}

	public void componentResized(ComponentEvent event)
	{
		if(event.getID()==event.COMPONENT_RESIZED) {
			//			Graphics g = getGraphics();
			//			g.setFont(font);
			//			FontMetrics fm = g.getFontMetrics();
			//			width = fm.charWidth(' ')*80 + xCharOffset*2 + props.getInteger("xFrame");
			//			height = fm.getHeight()*45 + yCharOffset*2 + props.getInteger("yFrame");
			Dimension dim = getSize();
			width = dim.width;
			height = dim.height;

			props.setProperty("scroll_lines",((int)(getMaxCharHeight(fixedFontHeight)*2/3))+"");
		}
	}


















	/**
	 * This closes the window and releases used resources (but doesn't necessarily dispose of them).
	 */
	public void dispose()
	{
		super.dispose();
		jerminal = null;
		jerminalIn = null;
		jerminalOut = null;
		keyListener = null;
		props = null;
	}

	/**
	 * This forces the window to repaint itself. The boolean parameter tells the method whether
	 * it should redraw the entire window. If refreshAll is set to true, the entire window will
	 * be redrawn, if it is set to false, only the line that the cursor is on will be redrawn.
	 */
	public void refresh(boolean refreshAll)
	{
		if(refreshAll) {
			repaint();
		}
		else {
			Graphics g = getGraphics();
			FontMetrics fm = g.getFontMetrics();
			int maxStringLength = getMaxCharWidth(fixedFontWidth);		
			
			//			int y = yoffset+yCharOffset+(charHeight*(getMaxCharHeight(charHeight)-1));
			int y = yoffset+yCharOffset+(fixedFontHeight*(getMaxCharHeight(fixedFontHeight)-(text.length-cursorY)));

			if(text[cursorY]!=null && text[cursorY].length()>maxStringLength) {
				int lines = (int)(text[cursorY].length()/maxStringLength);
				if(text[cursorY].length()%maxStringLength == 0) {
					lines--;
				}
				//System.out.println((y-(lines*charHeight))+","+(lines*charHeight));
				repaint(xCharOffset + xoffset, y-(lines*fixedFontHeight),width+10,(lines+1)*fixedFontHeight+yCharOffset);
			}
			else {
				repaint(xCharOffset + xoffset, y, width+25, fixedFontHeight+yCharOffset+25);
			}
		}

	}


	/**
	 * This method draws the Strings in the String array text[]. It starts at the bottom of the
	 * screen and starts with the last array element (unless the window is scrolling). It also draws
	 * the cursor. Lines that exceed the maximum number of characters that fit horizontally across the 
	 * current window will be wrapped.
	 */
	public void paint(Graphics g)
	{
		g.setFont(font);
		FontMetrics fm = g.getFontMetrics();

		//		g.setColor(bgColor);
		//		g.fillRect(0,0,width,height);
		g.setColor(fgColor);

		//System.out.println("height= "+getMaxCharHeight(charHeight));
		//System.out.println("y = "+y);

		int maxStringLength = getMaxCharWidth(fixedFontWidth);
		int maxHeight = fixedFontHeight*getMaxCharHeight(fixedFontHeight);
		if(scrolling) {
			int y = yoffset+yCharOffset;
			if(text!=null) {
				for(int x=scrollIndex;x<text.length;x++) {
					if(text[x] != null) {
						if(text[x].length() > maxStringLength) {
							int linesRequired = (int)(text[x].length()/maxStringLength);
							if(text[x].length()%maxStringLength > 0 ) {
								linesRequired++;
							}
							String tmp = null;
							String tmpString = text[x];
							for(int z=0;z<linesRequired;z++) {
								try {
									tmp = tmpString.substring(0,maxStringLength);
									tmpString = tmpString.substring(maxStringLength);
								}
								catch(StringIndexOutOfBoundsException adasf) {
									tmp = tmpString;
								}
								g.drawString(tmp,xoffset+xCharOffset,y);
								y = y+fixedFontHeight;
								if(y > (maxHeight+yoffset+yCharOffset)) {
									break;
								}
							}
						}
						else {
							g.drawString(text[x],xoffset+xCharOffset,y);
							y = y+fixedFontHeight;
							if(y > (maxHeight+yoffset+yCharOffset)) {
								break;
							}
						}
					}
				}
				if(y<=(maxHeight+yoffset+yCharOffset)) {
					scrolling = false;
					refresh(true);
				}
			}
		}
		else {
			int y = yoffset+yCharOffset+maxHeight;
			if(text!=null) {
				for(int x=text.length-1;x>=0;x--) {
					if(text[x]!=null) {
						if(text[x].length() > maxStringLength) {
							int linesRequired = (int)(text[x].length()/maxStringLength);
							if(text[x].length()%maxStringLength > 0 ) {
								linesRequired++;
							}
							String[] tmp = new String[linesRequired];
							String tmpString = text[x];
							for(int z=0;z<tmp.length;z++) {
								try {
									tmp[z] = tmpString.substring(0,maxStringLength);
									tmpString = tmpString.substring(maxStringLength);
								}
								catch(StringIndexOutOfBoundsException adasf) {
									tmp[z] = tmpString;
								}
							}
							for(int z=tmp.length-1;z>=0;z--) {
								g.drawString(tmp[z],xoffset+xCharOffset,y);
								y = y-fixedFontHeight;
								if(y<=(yoffset+yCharOffset)) { 
									break;
								}
							}
						}
						else {
							g.drawString(text[x],xoffset+xCharOffset,y);
						}

						y = y-fixedFontHeight;
						if(y<=(yoffset+yCharOffset)) {
							scrollIndex = x;
							break;
						}
					}
				}
				if(y>0) {
					scrollIndex = text.length-props.getInteger("scroll_lines");
				}

				y = yoffset+yCharOffset+(fixedFontHeight*(getMaxCharHeight(fixedFontHeight)-1));
				if(drawCursor) {
					g.setXORMode(bgColor);
					if(text[cursorY]!=null && text[cursorY].length()>maxStringLength) {

						int lines = (int)(text[cursorY].length()/maxStringLength);
						if(text[cursorY].length()%maxStringLength > 0) {
							lines++;
						}
						int line = (int)(cursorX/maxStringLength);
						if(line!=lines) {
							line++;
						}
						int col = (int)(cursorX%maxStringLength);
				
						if(cursorX == text[cursorY].length() && col==0 && cursorX!=0) {
							g.fillRect((maxStringLength-1)*fixedFontWidth + xCharOffset + xoffset + 1,
										  y - ((lines-line)*fixedFontHeight),
										  ///								  (cursorY-(lines-line))*charHeight + yCharOffset + yoffset,
										  (int)(fixedFontWidth/2),fixedFontHeight);
						} 
						else {
							g.fillRect(col*fixedFontWidth + xCharOffset + xoffset,
										  //								  (cursorY-(lines-line))*charHeight + yCharOffset + yoffset,
										  y - ((lines-line)*fixedFontHeight) + 2,
										  fixedFontWidth,fixedFontHeight);
						}
					}
					else {
						g.fillRect(cursorX*fixedFontWidth + xCharOffset + xoffset,
									  //							  cursorY*charHeight + yCharOffset + yoffset,
									  y + 2,
									  fixedFontWidth,fixedFontHeight);

					}
				}
			}
		}
	}

































	public void print(String s)
	{
		System.out.println("[JerminalWindow] "+s);
	}

	public void error(String s)
	{
		System.out.println("{JerminalWindow} "+s);
	}

	public void error(String s, Exception e)
	{
		System.out.println("{JerminalWindow} "+s+" : "+e);
		e.printStackTrace();
	}

	public static void main(String[] args)
	{
		/**
		try {
			PipedInputStream in = new PipedInputStream();
			PipedOutputStream jOut = new PipedOutputStream(in);
			PipedInputStream jIn = new PipedInputStream();
			PipedOutputStream out = new PipedOutputStream(jIn);

			JerminalWindow window = new JerminalWindow(new Props());

			EmacsKeyListener keys = new EmacsKeyListener(window);
			window.keyListener = keys;
			window.addKeyListener(window.keyListener);

			Props p = new Props();
			//			p.setProperty("local_echo","true");
			window.jerminal = new JerminalSession(jIn,jOut,p,true,true);
			window.jerminalIn = in;
			window.jerminalOut = out;
			window.jerminal.start();

			//			System.setErr(new PrintStream(jOut));
			//			System.setOut(new PrintStream(jOut));
			
			try {
				while(true) {
					boolean refreshAll = true;
					int i = in.read();
					char c = (char)i;
					if(i==10 || i==13) {
						window.makeNewLine();
						refreshAll = true;
					}
					else if(i==9) {
						//						String old = window.text[window.cursorY];
						//						window.moveCursorX(2);
						window.insertTab();
						//						refreshAll = !keys.lineDiff(old,window.text[window.cursorY]);
					}
					else {
						//						String old = window.text[window.cursorY];
						window.insertAtCursor(c);
						window.moveCursorX(1);
						//						refreshAll = !keys.lineDiff(old,window.text[window.cursorY]);
					}
					window.refresh(refreshAll);
				}
			}
			catch(Exception e) {
				e.printStackTrace();
			}
		}
		catch(Exception er) {
			er.printStackTrace();
		}
		*/
	}

	
}
