
package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;
import java.util.*;


/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This is the super class of all Jerminal commands. 
 */
public abstract class JerminalCommand
{
	/** This field stores the command as it should be given to the interpreter. This should be set to what the classname is. */
	public String commandString;
	/** This is the Jerminal which will be using the JerminalCommand */
	public JerminalSession jerminal;
	/** This is the help string */
	public String helpString;

	/**
	 * @param jerm This is the JerminalSession which will be using this command
	 */
	public JerminalCommand(JerminalSession jerm)
	{
		jerminal = jerm;
	}

	/**
	 * This method is called by evaluate to check for the help flag. 
	 * @param operands The list of operands to check for the help flag.
	 * @return Returns TRUE if there exists the help flag in the operands
	 */
	public boolean help(Vector operands)
	{
		boolean rval = false;
		try {
			for(int x=0;x<operands.size();x++) {
				if(operands.elementAt(x).toString().equals("--help")) {
					rval = true;
				}
			}
		}
		catch(Exception e) {;}
		return(rval);
	}

	/**
	 * This method is called by the JerminalSession when it uses this command.
	 * @param operands This Vector contains the operands (arguments) which were issued with the command.
	 * @return Returns the result of the evaluation.
	 */
	public abstract Object evaluate(Vector operands);

	public void print(String s)
	{
		jerminal.print(commandString+": "+s);
	}

	public void error(String s, Exception e)
	{
		jerminal.error(commandString+": "+s,e);
	}

	public String toString()
	{
		return(commandString);
	}
}
