
package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;
import java.util.*;
import java.io.*;
import kinetoscope.util.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This abstract subclass of JerminalCommand provides the functionality to resolve
 * UNIX style pathnames. Commands that require pathnames as its arguments or require 
 * the built in functionality of resolving pathnames should subclass this class.
 */
public abstract class UnixFileCommand extends JerminalCommand
{
	/** local copy of the path separator character */
	public char pathSeparator;

	/** 
	 * @param j This is the JerminalSession which will be using this command 
	 * @exception UnauthorizedAccessException Throws this exception if the file access for the JerminalSession is set to false
	 */
	public UnixFileCommand(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		if(!j.allowFileAccess()) {
			throw(new UnauthorizedAccessException("Access to the file system is not granted"));
		}

		pathSeparator = j.getProps().getString("path_separator_char").charAt(0);
	}

	

	/**
	 * This method returns the basename part of the pathname
	 */
	public String basename(String path)
	{
		if(path.lastIndexOf(pathSeparator)!=-1) {
			return(path.substring(0,path.lastIndexOf(pathSeparator)+1));
		}
		else {
			return("");
		}
	}

	/**
	 * This method returns the filename part of the pathname
	 */
	public String filename(String path)
	{
		if(path.lastIndexOf(pathSeparator)!=-1) {
			return(path.substring(path.lastIndexOf(pathSeparator)+1));
		}
		else {
			return(path);
		}
	}

	/**
	 * This method resolves paths which use ., .., and ~ symbols.
	 */
	public File resolve(String path)
	{
		/*
		String tmpPath = path;
		File rval = null;
		File pwd = (File)jerminal.getProps().getProperty("pwd");
		
		Vector v = new Vector();
		int index = tmpPath.indexOf(pathSeparator);
		while(index!=-1) {
			v.addElement(tmpPath.substring(0,index));
			tmpPath = tmpPath.substring(index+1);
			index = tmpPath.indexOf(pathSeparator);
		}
		if(tmpPath.trim().length()>0) {
			v.addELement(tmpPath.trim());
		}

		boolean done = false;
		while(!done) {
			for(int x=0;x<

		*/
		File rval = (File)jerminal.getProps().getProperty("pwd");
		try {
			if(path.startsWith("/")) {
				rval = new File(path);
			}
			else {
				if(path.indexOf("~")==0) {
					path = jerminal.getProps().getString("home_directory")+path.substring(1);
					rval = new File(new File(path).getCanonicalPath());
				}
				else {
					rval = new File(new File(rval,path).getCanonicalPath());
				}									
			}
		}
		catch(Exception e) {;}

		return(rval);
	}

}
