package com.cometway.jerminal.commands;

import java.util.*;

import com.cometway.jerminal.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command adds a classname alias to the current jerminal session
 * Synopsis:<br>
 * <dl>
 * <DT>aliasClass</DT>
 * <DD>Lists all the classname aliases.</DD><p>
 * <DT>aliasClass [alias]</DT>
 * <DD>Lists the classname alias who's name is [alias]</DD><p>
 * <DT>aliasClass [alias] [classname]</DT>
 * <DD>Adds an alias which sets the alias [alias] to the classname [classname].</DD><p>
 * </DL>
 * 
 */
public class aliasClass extends JerminalCommand
{
	public aliasClass(JerminalSession j)
	{
		super(j);
		commandString = "aliasClass";
		helpString = "This command adds a classname alias to the current jerminal session\n\n"+
			"Synopsis:\n"+
			"  aliasClass\n"+
			"Lists all the classname aliases.\n\n"+
			"  aliasClass [alias]\n"+
			"Lists the classname alias who's name is [alias]\n\n"+
			"  aliasClass [alias] [classname]"+
			"Adds an alias which sets the alias [alias] to the classname [classname].\n\n";
	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		if(operands.size()==0) {
			StringBuffer buffer = new StringBuffer();
			Enumeration e = jerminal.classAliases.keys();
			while(e.hasMoreElements()) {
				String key = (String)e.nextElement();
				buffer.append(key+" = "+jerminal.classAliases.get(key)+"\n");
			}
			rval = buffer.toString();
		}
		else if(operands.size()==1) {
			if(jerminal.classAliases.containsKey(operands.elementAt(0))) {
				rval = new String(operands.elementAt(0)+" = "+jerminal.classAliases.get(operands.elementAt(0))+"\n");
			}
			else {
				rval = new String("Class alias "+operands.elementAt(0)+" not found.\n");
			}
		}
		else {
			rval = jerminal.classAliases.put(operands.elementAt(0).toString(),operands.elementAt(1));
		}

		return(rval);
	}

}
