
package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;

import java.util.*;
import java.io.*;

import kinetoscope.util.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command allows you to change your current working directory.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>cd</DT>
 * <DD>This will change to your home directory, or what your <i>home_directory</i> variable is set to.</DD><p>
 * <DT>cd [pathname]</DT>
 * <DD>This will change your current working directory to the one specified by [pathname].</DD><p>
 * </DL>
 *
 */
public class cd extends UnixFileCommand
{
	public cd(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		commandString = "cd";
		helpString = "This command allows you to change your current working directory.\n\n"+
			"Synopsis:\n\n"+
			"  cd\n"+
			"This will change to your home directory, or what your 'home_directory' variable is set to.\n\n"+
			"  cd [pathname]\n"+
			"This will change your current working directory to the one specified by [pathname].\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		Props p = jerminal.getProps();
		try {
			if(operands.size()==0) {
				p.setProperty("pwd",new File(p.getString("home_directory")));
			}
			else if(operands.elementAt(0).equals(".")) {
				;
			}
			else if(operands.elementAt(0).equals("..")) {
				File pwd = (File)p.getProperty("pwd");
				p.setProperty("pwd",new File(pwd.getParent()));
			}
			else {
				File pwd = (File)p.getProperty("pwd");
				File dest = resolve((String)operands.elementAt(0));
				if(dest.exists()) {
					p.setProperty("pwd",dest);
				}
				else {
					rval = new String("cd: "+dest+" directory not found\n");
				}
			}
		}
		catch(Exception e) {
			error("Could not change directory from "+p.getString("pwd"),e);
		}

		return(rval);
	}

}
