package com.cometway.jerminal.commands;

import java.util.*;
import java.lang.reflect.*;

import com.cometway.jerminal.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command takes a java.lang.Object and a field name (java.lang.String). This command returns the 
 * field of the Object that was given as an argument.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>getField [Object] [field name]</DT>
 * <DD>The field [field name] of the object [Object] is returned. It will be the same type (the same
 * Class) as the field.</DD><p>
 * </DL>
 */
public class getField extends JerminalCommand
{
	public getField(JerminalSession j)
	{
		super(j);
		commandString = "getField";
		helpString = "This command takes a java.lang.Object and a field name (java.lang.String). This command returns the "+
			"field of the Object that was given as an argument.\n\n"+
			"Synopsis:\n"+
			"  getField [Object] [field name]\n"+
			"The field [field name] of the object [Object] is returned. It will be the same type (the same"+
			"Class) as the field.\n\n";
	}


	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		try {
			Object o = operands.elementAt(0);
			String fieldName = (String)operands.elementAt(1);
			Field f = o.getClass().getField(fieldName);
			rval = f.get(o);				
		}
		catch(Exception e) {
			error("Error in evaluate",e);
		}
		return(rval);
	}



}
