package com.cometway.jerminal.commands;


import java.util.*;
import java.lang.reflect.*;

import com.cometway.jerminal.*;

import kinetoscope.util.*;


/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command instantiates a Class and returns the new instance of that class.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>instantiate [classname] [constructor param1 type] (constructor param2 type)... [constructor param1 value] (constructor param2 value)...</DT>
 * <DD>The class [classname] is to be instantiated. The constructor to use will have as its first argument
 * an Object of type [constructor param1 type] and if exists, the second argument will be of type 
 * (constructor param2 type) and so on. There can be as many types as required to describe the 
 * constructor. The returned object will be of type [classname] and be instantiated with the arguments:
 * [constructor param1 value] and if exists, (constructor param2 value) and so on. The number of 
 * constructor values must equal the number of constructor types. The constructor values can be casted
 * into the primitive Object representations. For example [constructor param1 value] could be 
 * '(Integer)57'.</DD><p>
 * </DL>
 */
public class instantiate extends JerminalCommand
{
	public instantiate(JerminalSession j)
	{
		super(j);
		commandString = "instantiate";
		helpString = "This command instantiates a Class and returns the new instance of that class.\n\n"+
			"Synopsis:\n"+
			"  instantiate [classname] [constructor param1 type] (constructor param2 type)... [constructor param1 value] (constructor param2 value)...\n"+
			"The class [classname] is to be instantiated. The constructor to use will have as its first argument "+
			"an Object of type [constructor param1 type] and if exists, the second argument will be of type "+
			"(constructor param2 type) and so on. There can be as many types as required to describe the "+
			"constructor. The returned object will be of type [classname] and be instantiated with the arguments: "+
			"[constructor param1 value] and if exists, (constructor param2 value) and so on. The number of "+
			"constructor values must equal the number of constructor types. The constructor values can be casted "+
			"into the primitive Object representations. For example [constructor param1 value] could be "+
			"'(Integer)57'.\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		try {
			//			if(((int)(operands.size()/2))*2 < operands.size()) {
			//				print("Hmm... you do not have the same number of types as you do objects.");
			//			}
			int offset = (int)((operands.size()-1)/2);
			Class[] paramTypes = new Class[offset];
			Object[] params = new Object[offset];

			for(int x=0;x<paramTypes.length;x++) {
				paramTypes[x] = Class.forName(jerminal.getAliasedClass((String)operands.elementAt(x+1)));
				Object o = operands.elementAt(x+offset+1);
				if(o instanceof String) {
					String test = (String)o;
					if(test.startsWith("(") && test.indexOf(")")!=-1) {
						String castString = test.substring(1,test.indexOf(")"));
						test = test.substring(test.indexOf(")")+1);
						if(castString.equals("String")) {
							params[x] = (String)test;
						}
						else if(castString.equals("Integer")) {
							params[x] = new Integer(test);
						}
						else if(castString.equals("Float")) {
							params[x] = new Float(test);
						}
						else if(castString.equals("Boolean")) {
							params[x] = new Boolean(test);
						}
						else if(castString.equals("Byte")) {
							params[x] = new Byte(test);
						}
						else if(castString.equals("Character")) {
							params[x] = new Character(test.charAt(0));
						}
						else if(castString.equals("Double")) {
							params[x] = new Double(test);
						}
						else if(castString.equals("Long")) {
							params[x] = new Long(test);
						}
						else if(castString.equals("Short")) {
							params[x] = new Short(test);
						}
						else if(castString.equals("StringBuffer")) {
							params[x] = new StringBuffer(test);
						}
					}
					else {
						params[x] = operands.elementAt(x+offset+1);
					}
				}
				else {
					params[x] = operands.elementAt(x+offset+1);
				}
			}

			Class newClass = Class.forName(jerminal.getAliasedClass((String)operands.elementAt(0)));
			Constructor con = newClass.getConstructor(paramTypes);
			rval = con.newInstance(params);
			print("Created new instance of "+newClass.getName());
		}
		catch(Exception e) {
			error("Could not instantiate class",e);
		}

		return(rval);
	}





}
