package com.cometway.jerminal.commands;


import java.util.*;
import java.lang.reflect.*;

import com.cometway.jerminal.*;

import kinetoscope.util.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command invokes a method on an existing Object.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>invoke [Target Object] [method name] [method param1 type] (method param2 type)... [method param1 value] (method param2 value)...</DT>
 * <DD>The Object [Target Object] will have the method [method name] invoked. The Method will require as
 * its arguments the types [method param1 type] and if exists, (method param2 type) and so on. There can
 * be as many types as needed to describe the method. The command returns the Object returned by the
 * method invocation when given the arguments [method param1 value] and if exists (method param2 type)
 * and so on. The number of method values must equal to the number of method types. The method values
 * can be casted into primitive Object representations. For example, [method param1 value] could be
 * '(Character)c'. If the [Target Object] is a classname, the method invoked is assumed to be a static method.</DD><p>
 * </DL>
 */
public class invoke extends JerminalCommand
{
	public invoke(JerminalSession j)
	{
		super(j);
		commandString = "invoke";
		helpString = "This command invokes a method on an existing Object.\n\n" +
			"Synopsis:\n"+
			"  invoke [Target Object] [method name] [method param1 type] (method param2 type)... [method param1 value] (method param2 value)...\n"+
			"The Object [Target Object] will have the method [method name] invoked. The Method will require as " +
			"its arguments the types [method param1 type] and if exists, (method param2 type) and so on. There can " +
			"be as many types as needed to describe the method. The command returns the Object returned by the " +
			"method invocation when given the arguments [method param1 value] and if exists (method param2 type) " +
			"and so on. The number of method values must equal to the number of method types. The method values " +
			"can be casted into primitive Object representations. For example, [method param1 value] could be " +
			"'(Character)c'. If the [Target Object] is a classname, the method invoked is assumed to be a static method.\n\n";
	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		try {
			Object object = operands.elementAt(0);
			if(object.toString().equals("null")) {
				object = null;
			}
			String methodString = (String)operands.elementAt(1);

			int offset = (int)((operands.size()-2)/2);

			Class[] paramTypes = new Class[offset];
			Object[] params = new Object[offset];

			for(int x=0;x<params.length;x++) {
				paramTypes[x] = Class.forName(jerminal.getAliasedClass((String)operands.elementAt(x+2)));
				Object o = operands.elementAt(x+offset+2);
				if(o instanceof String) {
					String test = (String)operands.elementAt(x+offset+2);
					if(test.startsWith("(") && test.indexOf(")")!=-1) {
						String castString = test.substring(1,test.indexOf(")"));
						test = test.substring(test.indexOf(")")+1);
						if(castString.equals("String")) {
							params[x] = (String)test;
						}
						else if(castString.equals("Integer")) {
							params[x] = new Integer(test);
						}
						else if(castString.equals("Float")) {
							params[x] = new Float(test);
						}
						else if(castString.equals("Boolean")) {
							params[x] = new Boolean(test);
						}
						else if(castString.equals("Byte")) {
							params[x] = new Byte(test);
						}
						else if(castString.equals("Character")) {
							params[x] = new Character(test.charAt(0));
						}
						else if(castString.equals("Double")) {
							params[x] = new Double(test);
						}
						else if(castString.equals("Long")) {
							params[x] = new Long(test);
						}
						else if(castString.equals("Short")) {
							params[x] = new Short(test);
						}
						else if(castString.equals("StringBuffer")) {
							params[x] = new StringBuffer(test);
						}
					}
					else {
						params[x] = operands.elementAt(x+offset+2);
					}
				}
				else {
					params[x] = operands.elementAt(x+offset+2);
				}
			}
			

			
			Class clas = null;
			try {
				clas = Class.forName(jerminal.getAliasedClass(object.toString()));
				object = null;
			}
			catch(Exception e) {
				clas = object.getClass();
			}
			Method method = clas.getMethod(methodString,paramTypes);
			rval = method.invoke(object,params);
		}
		catch(Exception e) {
			error("Could not invoke method",e);
		}

		return(rval);
	}
}
