package com.cometway.jerminal.commands;


import java.util.*;
import java.lang.reflect.*;


import com.cometway.jerminal.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command takes classnames as its arguments and lists the class information, including its
 * subclasses, interfaces, constructors, fields, and methods. 
 */
public class javap extends JerminalCommand
{

	public javap(JerminalSession j) 
	{
		super(j);
		commandString = "javap";
		helpString = "This command is the java class viewer\n\n";
	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		try {
			StringBuffer buffer = new StringBuffer();
			for(int x=0;x<operands.size();x++) {
				Class clas = Class.forName(jerminal.getAliasedClass((String)operands.elementAt(x)));
				String classname = clas.getName();

				if(clas.isInterface()) {
					buffer.append("interface ");
					buffer.append(clas.getName());
				}
				else {
					buffer.append("class ");
					buffer.append(clas.getName());
					buffer.append(" extends ");
					buffer.append(clas.getSuperclass().getName());
				}
				Class[] interfaces = clas.getInterfaces();
				if(interfaces.length>0) {
					buffer.append(" implements ");
					buffer.append(interfaces[0].getName());
					for(int y=1;y<interfaces.length;y++) {
						buffer.append(", ");
						buffer.append(interfaces[y].getName());
					}
				}
				buffer.append("\n");

				Constructor[] cons = clas.getConstructors();
				for(int y=0;y<cons.length;y++) {
					String s = cons[y].toString();
					buffer.append(s);
					buffer.append("\n");
				}

				Field[] fields = clas.getFields();
				for(int y=0;y<fields.length;y++) {
					String s = fields[y].toString();
					int i = s.indexOf(classname+".");
					if(i!=-1) {
						s = s.substring(0,i)+s.substring(i+classname.length()+1);
						buffer.append(s);
						buffer.append("\n");
					}
				}
				Method[] methods = clas.getMethods();
				for(int y=0;y<methods.length;y++) {
					String s = methods[y].toString();
					int i = s.indexOf(classname+".");
					if(i!=-1) {
						s = s.substring(0,i)+s.substring(i+classname.length()+1);
						buffer.append(s);
						buffer.append("\n");
					}
				}
				buffer.append("\n");
			}
			rval = buffer.toString();
		}
		catch(Exception e) {
			error("Error in evaluate()",e);
		}
		return(rval);
	}
}
