package com.cometway.jerminal.commands;

import java.util.*;
import java.io.*;
import java.lang.reflect.Constructor;

import kinetoscope.util.*;

import com.cometway.jerminal.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command is similar to the <b>instantiate</b> command. But instead of using the default
 * ClassLoader to load the Class, the loadClass command uses its own ClassLoader and reloads the 
 * class data. 
 * <DL>
 * <DT>loadClass [pathname] (constructor param1 type)... (constructor param1 value)...</DT>
 * <DD>The [pathname] is the path name of the .class file of the Class to load. This class is loaded
 * and the constructor that mathces the (constructor param1 type) types is loaded. There can be as many 
 * types as needed to describe the constructor. The command returns the new instance of the Object
 * instantiated from the class data loaded from the [pathname]. The (constructor param1 value) values
 * are used as arguments for the constructor. The number of types must equal the number of values.
 * </DD><p>
 * </DL>
 */
public class loadClass extends UnixFileCommand
{
	public loadClass(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		commandString = "loadClass";
		helpString = "This command is similar to the 'instantiate' command. But instead of using the default "+
			"ClassLoader to load the Class, the loadClass command uses its own ClassLoader and reloads the "+
			"class data. \n\n"+
			"Synopsis: \n"+
			"  loadClass [pathname] (constructor param1 type)... (constructor param1 value)...\n"+
			"The [pathname] is the path name of the .class file of the Class to load. This class is loaded "+
			"and the constructor that mathces the (constructor param1 type) types is loaded. There can be as many "+
			"types as needed to describe the constructor. The command returns the new instance of the Object "+
			"instantiated from the class data loaded from the [pathname]. The (constructor param1 value) values "+
			"are used as arguments for the constructor. The number of types must equal the number of values. \n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		try {
			File f = resolve((String)operands.elementAt(0));
			if(!f.exists()) {
				error("Could not find the class definition for "+operands.elementAt(0),null);
			}
			else {
				InputStream in = null;
				byte[] data = null;
				try {
					in = new FileInputStream(f);
					data = new byte[(int)f.length()];
					in.read(data);
				}
				catch(Exception e) {
					error("Error reading class data",e);
				}
				try {
					in.close();
				}
				catch(Exception e) {;}
				
				com.cometway.jerminal.ClassLoader loader = new com.cometway.jerminal.ClassLoader();
				Class clas = loader.loadClass(data);
				
				int offset = (int)((operands.size()-1)/2);
				Class[] types = new Class[offset];
				Object[] params = new Object[offset];
				for(int x=0;x<offset;x++) {
					types[x] = Class.forName(jerminal.getAliasedClass((String)operands.elementAt(x+1)));
					params[x] = operands.elementAt(offset+x+1);
				}
				
				Constructor con = clas.getConstructor(types);
				rval = con.newInstance(params);
				print("Class loaded");
			}
		}
		catch(Exception e) {
			error("Error in evaluate",e);
		}
		return(rval);
	}

}
