
package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;

import java.util.*;
import java.io.*;

import kinetoscope.util.*;
import kinetoscope.io.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command lists files and directories. There are two flags which you can use: the <i>l</i> and the 
 * <i>a</i> flags (flags are preceded by a '-' character). The <i>l</i> flag will cause this command to
 * give to a long (detailed) listing of the files and directories. The <i>a</i> flag will display
 * hidden files as well (files starting with the '.' character).
 * Synopsis:<br>
 * <DL>
 * <DT>ls (flags)</DT>
 * <DD>Displays the contents of your current working directory.</DD><p>
 * <DT>ls (flags) [pathname1] (pathname2) (pathname3) ...</DT>
 * <DD>Displays the contents of [pathname1], and if they exist, the contents of (pathname2) and
 * (pathname3). All the arguments given to the ls command will be interpreted as pathnames.</DD><p>
 * </DL>
 *
 */
public class ls extends UnixFileCommand
{
	public ls(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		commandString = "ls";
		helpString = "This command lists files and directories. There are two flags which you can use: the 'l' and the "+
			"'a' flags (flags are preceded by a '-' character). The 'l; flag will cause this command to "+
			"give to a long (detailed) listing of the files and directories. The 'a' flag will display "+
			"hidden files as well (files starting with the '.' character). "+
			"Synopsis:\n"+
			"  ls (flags)\n"+
			"Displays the contents of your current working directory.\n\n"+
			"  ls (flags) [pathname1] (pathname2) (pathname3) ...\n"+
			"Displays the contents of [pathname1], and if they exist, the contents of (pathname2) and "+
			"(pathname3). All the arguments given to the ls command will be interpreted as pathnames.\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		Props p = jerminal.getProps();
		try {
			if(operands.size()==0) {
				File pwd = (File)p.getProperty("pwd");
				String[] list = pwd.list();
				StringBuffer buffer = new StringBuffer();
				for(int x=0;x<list.length;x++) {
					buffer.append(list[x]);
					buffer.append("\n");
				}
				rval = buffer.toString();
			}
			else {
				boolean longList = false;
				boolean hiddenFiles = false;
				Vector v = new Vector();

				for(int x=0;x<operands.size();x++) {
					String op = (String)operands.elementAt(x);
					if(op.equals("-l")) {
						longList = true;
					}
					else if(op.equals("-a")) {
						hiddenFiles = true;
					}
					else if(op.equals("-al") || op.equals("-la")) {
						longList = true;
						hiddenFiles = true;
					}
					else {
						//System.out.println(filename(op)+" "+File.pathSeparatorChar);
						if(filename(op).indexOf("*")!=-1) {
							File tmpF = resolve(basename(op));
							if(tmpF.exists()) {
								String[] files = tmpF.list(new WildCardFilenameFilter(filename(op)));
								for(int y=0;y<files.length;y++) {
									v.addElement(resolve(basename(op)+jerminal.getProps().getString("path_separator")+files[y]));
								}
							}
							else {
							   error("Could not resolve: "+op,null);
							}
						}
						else {							
							v.addElement(resolve(op));
						}
					}
				}
				
				File pwd = (File)p.getProperty("pwd");
				if(v.size()==0) {
					StringBuffer buffer = new StringBuffer();
					String[] files = pwd.list();
					if(longList) {
						for(int x=0;x<files.length;x++) {
							if(hiddenFiles || (!files[x].startsWith("."))) {
								File f = new File(pwd,files[x]);
								if(f.isDirectory()) {
									buffer.append("d");
								}
								else {
									buffer.append("-");
								}
								if(f.canRead()) {
									buffer.append("r");
								}
								else {
									buffer.append("-");
								}
								if(f.canWrite()) {
									buffer.append("w");
								}
								else {
									buffer.append("-");
								}
								buffer.append("\t");
								buffer.append(f.length());
								buffer.append("\t");
								buffer.append((new Date(f.lastModified())).toString());
								buffer.append("\t");
								buffer.append(files[x]);
								buffer.append("\n");
							}
						}
					}
					else {
						for(int x=0;x<files.length;x++) {
							if(hiddenFiles || (!hiddenFiles && !files[x].startsWith("."))) {
								buffer.append(files[x]);
								buffer.append("\n");
							}
						}
					}
					rval = buffer.toString();
				}
				else {
					StringBuffer buffer = new StringBuffer();
					for(int z=0;z<v.size();z++) {
						File tmpFile = (File)v.elementAt(z);
						if(tmpFile.isDirectory()) {
							String[] files = tmpFile.list();
							if(longList) {
								for(int x=0;x<files.length;x++) {
									if(hiddenFiles || (!hiddenFiles && !files[x].startsWith("."))) {
										File f = new File(tmpFile,files[x]);
										if(f.isDirectory()) {
											buffer.append("d");
										}
										else {
											buffer.append("-");
										}
										if(f.canRead()) {
											buffer.append("r");
										}
										else {
											buffer.append("-");
										}
										if(f.canWrite()) {
											buffer.append("w");
										}
										else {
											buffer.append("-");
										}
										buffer.append("\t");
										buffer.append(f.length());
										buffer.append("\t");
										buffer.append((new Date(f.lastModified())).toString());
										buffer.append("\t");
										buffer.append(files[x]);
										buffer.append("\n");
									}
								}
							}
							else {
								for(int x=0;x<files.length;x++) {
									if(hiddenFiles || (!hiddenFiles && !files[x].startsWith("."))) {
										buffer.append(files[x]);
										buffer.append("\n");
									}
								}
							}
						}
						else {
							if(longList) {
								if(tmpFile.isDirectory()) {
									buffer.append("d");
								}
								else {
									buffer.append("-");
								}
								if(tmpFile.canRead()) {
									buffer.append("r");
								}
								else {
									buffer.append("-");
								}
								if(tmpFile.canWrite()) {
									buffer.append("w");
								}
								else {
									buffer.append("-");
								}
								buffer.append("\t");
								buffer.append(tmpFile.length());
								buffer.append("\t");
								buffer.append((new Date(tmpFile.lastModified())).toString());
								buffer.append("\t");
								buffer.append(tmpFile.getCanonicalPath());
								buffer.append("\n");
							}
							else {
								buffer.append(tmpFile.toString());
								buffer.append("\n");
							}
						}
					}
					rval = buffer.toString();
				}
			}
		}
		catch(Exception e) {
			error("Could not list directory "+p.getString("pwd"),e);
		}

		return(rval);
	}

}
