
package com.cometway.jerminal.commands;

import java.util.*;
import java.lang.reflect.*;
import com.cometway.jerminal.*;
/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command invokes the public static void main(String[]) method for the given class.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>main [class] (String arguments)</DT>
 * <DD>The public static void main(String[]) static method is invoked on the class [class]. If any
 * other operands are given, they are assumed to be Strings and will make up the String array that
 * is passed into the main method as arguments.</DD>
 * </DL>
 *
 *
 */
public class main extends JerminalCommand
{
	public main(JerminalSession j)
	{
		super(j);
		commandString = "main";
		helpString = "This command invokes the public static void main(String[]) method for the given class.\n\n"+
			"Synopsis:\n"+
			"  main [class] (String arguments)\n"+
			"The public static void main(String[]) static method is invoked on the class [class]. If any "+
			"other operands are given, they are assumed to be Strings and will make up the String array that "+
			"is passed into the main method as arguments.\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		
		try {
			Method mainMethod = null;
			Method[] methods = Class.forName(jerminal.getAliasedClass((String)operands.elementAt(0))).getMethods();
			String[] args = new String[operands.size()-1];
			for(int x=0;x<args.length;x++) {
				args[x] = (String)operands.elementAt(x+1);
			}
			for(int x=0;x<methods.length;x++) {
				if(methods[x].getName().equals("main")) {
					mainMethod = methods[x];
					break;
				}
			}
			if(mainMethod==null) {
				rval = "Class "+operands.elementAt(0)+" has no static main method";
			}
			else {
				Object[] methodArgs = new Object[1];
				methodArgs[0]=args;
				mainMethod.invoke(null,methodArgs);
				rval = "Executed "+operands.elementAt(0)+"'s static main method";
			}
		}
		catch(Exception e) {
			error("Could not invoke the static main method",e);
		}
		return(rval);

	}


}
