package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;

import java.util.*;
import java.io.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command will create a directory on the local file system.<p>
 * Synopsis:<p>
 * mkdir [directory name]<p>
 */
public class mkdir extends UnixFileCommand
{
	public mkdir(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		commandString = "mkdir";
		helpString = "This command will create a directory on the local file system.\n\n"+
			"Synopsis:\n"+
			"  mkdir [directory name]\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		
		try {
			if(operands.size()==0) {
				rval = new String("Invalid argument\n");
			}
			else {
				File newDir = resolve(operands.elementAt(0).toString());
				if(newDir.exists()) {
					rval = new String("Directory "+newDir+" already exists.");
				}
				else {
					if(!newDir.mkdir()) {
						rval = new String("Could not make directory: "+newDir);
					}
				}
			}
		}
		catch(Exception e) {
			error("Could not create directory",e);
		}

		return(rval);
	}
}
