package com.cometway.jerminal.commands;


import java.util.*;

import com.cometway.jerminal.*;

import kinetoscope.util.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command clears the list of cached commands. Since jerminal commands are Objects and need
 * to be instantiated in order to use them, the jerminal keeps a cache of the command Objects.
 * The <i>rehash</I> command will clear that cache and the command Objects are recached when they
 * are issued to the jerminal.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>rehash</DT>
 * <DD>This clears the entire cache.</DD><p>
 * <DT>rehash [command name]</DT>
 * <DD>This clears the cached command [command name] from the cache.</DD><p>
 * <DT>rehash [command name] [command Object]</DT>
 * <DD>This clears the cached command [command name] and recaches it using the command Object
 * [command Object]. This command assumes that you are passing a JerminalCommand of the appropriate
 * class, otherwise, problems may result.</DD><p>
 * </DL>
 */
public class rehash extends JerminalCommand
{

	public rehash(JerminalSession j)
	{
		super(j);
		commandString = "rehash";
		helpString = "This command clears the list of cached commands. Since jerminal commands are Objects and need "+
			"to be instantiated in order to use them, the jerminal keeps a cache of the command Objects. "+
			"The 'rehash' command will clear that cache and the command Objects are recached when they "+
			"are issued to the jerminal.\n\n"+
			"Synopsis:<br>\n"+
			"  rehash\n"+
			"This clears the entire cache.\n\n"+
			"  rehash [command name]\n"+
			"This clears the cached command [command name] from the cache.\n\n"+
			"  rehash [command name] [command Object]\n"+
			"This clears the cached command [command name] and recaches it using the command Object "+
			"[command Object]. This command assumes that you are passing a JerminalCommand of the appropriate "+
			"class, otherwise, problems may result.\n\n";
	}

	public Object evaluate(Vector operands)
	{
		if(help(operands)) {
			return(helpString+"\n");
		}
		if(operands.size()==0) {
			jerminal.cachedCommands = new Hashtable();
		}
		else if(operands.size()==1) {
			jerminal.cachedCommands.remove((String)operands.elementAt(0));
		}
		else if(operands.size()==2) {
			jerminal.cachedCommands.put((String)operands.elementAt(0),operands.elementAt(1));
		}

		return(null);
	}



}
