package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;

import java.util.*;
import java.io.*;

import kinetoscope.io.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command removes files and directories from the local file system. There are two flags which
 * the rm command will accept: the <i>r</i> flag and the <i>i</i> flag. The <i>r</i> flag tells the
 * rm command to recurse through directories and subdirectories when removing (this could be 
 * dangerous!). The <i>i</i> flag will make the rm command ask for a confirmation before deleting
 * anything.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>rm (flags) [pathname1] (pathname2) (pathname3) ...</DT>
 * <DD>This removes [pathname1] and if they exist, (pathname2) and (pathname3). The rm command will
 * assume all the arguments are pathnames and are to be removed. </DD><p>
 * </DL>
 */
public class rm extends UnixFileCommand
{
	public rm(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		commandString = "rm";
		helpString = "This command removes files and directories from the local file system. There are two flags which "+
			"the rm command will accept: the <i>r</i> flag and the <i>i</i> flag. The 'r' flag tells the "+
			"rm command to recurse through directories and subdirectories when removing (this could be "+
			"dangerous!). The 'i' flag will make the rm command ask for a confirmation before deleting "+
			"anything.\n\n"+
			"Synopsis:\n"+
			"  rm (flags) [pathname1] (pathname2) (pathname3) ...\n"+
			"This removes [pathname1] and if they exist, (pathname2) and (pathname3). The rm command will "+
			"assume all the arguments are pathnames and are to be removed. \n\n";

	}


	protected void removeDir(File dir, boolean i)
	{
		OutputStream out = jerminal.getOutputStream();
		InputStream in = jerminal.getInputStream();

		if(dir.isDirectory()) {
			String[] files = dir.list();
			if(files.length>0) {
				for(int x=0;x<files.length;x++) {
					File f = new File(dir,files[x]);
					if(f.isDirectory()) {
						removeDir(f,i);
					}
					else {
						if(i) {
							try {
								out.write(("remove "+f+"? (y/n) ").getBytes());
								out.flush();
								char q = (char)in.read();
								in.read();
								if(q=='y') {
									f.delete();
								}
							}
							catch(Exception e) {
								error("Could not delete file "+f,null);
							}
						}
						else {
							try {
								f.delete();
							}
							catch(Exception e) {
								error("Could not delete file "+f,null);
							}
						}
					}
				}
			}

			if(i) {
				try {
					out.write(("remove "+dir+"? (y/n) ").getBytes());
					out.flush();
					char q = (char)in.read();
					in.read();
					if(q=='y') {
						dir.delete();
					}
				}
				catch(Exception e) {
					error("Could not delete directory "+dir,null);
				}
			}
			else {
				try {
					dir.delete();
				}
				catch(Exception e) {
					error("Could not delete directory "+dir,null);
				}
			}
		}
		else {
			if(i) {
				try {
					out.write(("remove "+dir+"? (y/n) ").getBytes());
					out.flush();
					char q = (char)in.read();
					in.read();
					if(q=='y') {
						dir.delete();
					}
				}
				catch(Exception e) {
					error("Could not delete file "+dir,null);
				}
			}
			else {
				try {
					dir.delete();
				}
				catch(Exception e) {
					error("Could not delete file "+dir,null);
				}
			}
		}
	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		
		try {
			if(operands.size()==0) {
				rval = new String("Invalid argument\n");
			}
			else {
				boolean recursive = false;
				boolean interactive = false;

				Vector v = new Vector();
				for(int x=0;x<operands.size();x++) {
					String op = (String)operands.elementAt(x);
					if(op.equals("-r")) {
						recursive = true;
					}
					else if(op.equals("-i")) {
						interactive = true;
					}
					else if(op.equals("-ri")) {
						recursive = true;
						interactive = true;
					}
					else if(op.equals("-ir")) {
						recursive = true;
						interactive = true;
					}
					else {
						if(filename(op).indexOf("*")!=-1) {
							File tmpF = resolve(basename(op));
							if(tmpF.exists()) {
								String[] files = tmpF.list(new WildCardFilenameFilter(filename(op)));
								for(int y=0;y<files.length;y++) {
									v.addElement(resolve(basename(op)+jerminal.getProps().getString("path_separator")+files[y]));
								}
							}
							else {
							   error("Could not resolve: "+op,null);
							}
						}
						else {							
							v.addElement(resolve(op));
						}
					}
				}

				for(int x=0;x<v.size();x++) {
					File delFile = (File)v.elementAt(x);
					if(delFile.isDirectory() && recursive) {
						removeDir(delFile,interactive);
					}
					else if(delFile.isDirectory()) {
						error(delFile+" is a directory. It will not be deleted nor will the files in it.",null);
					}
					else {
						delFile.delete();
					}
				}
			}
		}
		catch(Exception e) {
			error("Could not create directory",e);
		}

		return(rval);
	}
}
