package com.cometway.jerminal.commands;

import java.util.*;
import java.lang.reflect.*;

import com.cometway.jerminal.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command sets a field of a java.lang.Object.<p>
 * Synopsis:<br>
 * <DL>
 * <DT>setField [Target Object] [field name] [Field Value Object] (field type)</DT>
 * <DD>The Object [Target Object]'s field, [field name] will be set to [Field Value Object]. In the
 * case that the field is a primitive, the (field type) is used to indicate which primitive to set
 * the field to.</DD><p>
 * </DL>
 */
public class setField extends JerminalCommand
{
	public setField(JerminalSession j)
	{
		super(j);
		commandString = "setField";
		helpString = "This command sets a field of a java.lang.Object.\n\n"+
			"Synopsis:\n"+
			"  setField [Target Object] [field name] [Field Value Object] (field type)\n"+
			"The Object [Target Object]'s field, [field name] will be set to [Field Value Object]. In the "+
			"case that the field is a primitive, the (field type) is used to indicate which primitive to set "+
			"the field to.\n\n";

	}


	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		try {
			Object o = operands.elementAt(0);
			String fieldName = (String)operands.elementAt(1);
			Object value = operands.elementAt(2);
			String castString = "";
			if(operands.size()>3) {
				castString = (String)operands.elementAt(3);
			}

			Class tmpClass = o.getClass();
			Field field = tmpClass.getField(fieldName);

			if(castString.equals("int")) {
				int val = 0;
				if(value.toString().trim().startsWith("-")) {
					val = -1*Integer.parseInt(value.toString().trim().substring(1));
				}
				else {
					val = Integer.parseInt((String)value);
				}
				field.setInt(o,val);
			}
			else if(castString.equals("float")) {
				field.setFloat(o,(new Float(value.toString())).floatValue());
			}
			else if(castString.equals("boolean")) {
				field.setBoolean(o,(new Boolean(value.toString())).booleanValue());
			}
			else if(castString.equals("byte")) {
				field.setByte(o,(new Byte(value.toString())).byteValue());
			}
			else if(castString.equals("char")) {
				field.setChar(o,value.toString().charAt(0));
			}
			else if(castString.equals("double")) {
				field.setDouble(o,(new Double(value.toString())).doubleValue());
			}
			else if(castString.equals("long")) {
				field.setLong(o,(new Long(value.toString())).longValue());
			}
			else if(castString.equals("short")) {
				field.setShort(o,(new Short(value.toString())).shortValue());
			}
			else {
				field.set(o,value);
			}
		}
		catch(Exception e) {
			error("Error in evaluate",e);
		}
		return(rval);
	}



}
