
package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;
import kinetoscope.util.*;
import java.util.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */


/**
 * This command displays and sets the environment variables of a jerminal session.
 * Synopsis:<br>
 * <DT>
 * <DT>setenv</DT>
 * <DD>This displays all the environment variables (in case sensitive alpha-numeric order).</DD><p>
 * <DT>setenv [variable name]</DT>
 * <DD>This displays the environment variable who's name is [variable name].</DD><p>
 * <DT>setenv [variable name] [value]</DT>
 * <DD>This sets the environment variable [variable name] to the value [value]. The old value is returned
 * by this call.</DD><p>
 * </DL>
 */
public class setenv extends JerminalCommand
{
	public setenv(JerminalSession j)
	{
		super(j);
		commandString = "setenv";
		helpString = "This command displays and sets the environment variables of a jerminal session.\n\n"+
			"Synopsis:\n"+
			"  setenv\n"+
			"This displays all the environment variables (in case sensitive alpha-numeric order).\n\n"+
			"  setenv [variable name]\n"+
			"This displays the environment variable who's name is [variable name].\n\n"+
			"  setenv [variable name] [value]\n"+
			"This sets the environment variable [variable name] to the value [value]. The old value is returned "+
			"by this call.\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}

		if(operands.size()>2) {
			print("There are "+(operands.size()-2)+" extra parameters given to the "+commandString+" command, they will be ignored");
		}
		else if(operands.size()<2) {
			if(operands.size()==0) {
				Props p = jerminal.getProps();
				Vector pairs = new Vector();
				Enumeration keys = p.enumerateKeys();
				while(keys.hasMoreElements()) {
					String key = (String)keys.nextElement();
					try {
						Pair pair = new Pair(key,p.getString(key));
						pairs.addElement(pair);
					}
					catch(Exception e) {
						pairs.addElement(new Pair(key,null));
					}
				}
				pairs = StringTools.pairCaseSensitiveSort(pairs);

				StringBuffer buffer = new StringBuffer();
				for(int x=0;x<pairs.size();x++) {
					Pair pair = (Pair)pairs.elementAt(x);
					buffer.append(pair.first().toString());
					buffer.append("=");
					buffer.append(pair.second()+"");
					buffer.append("\n");
				}

				return(buffer.toString());
			}
			else {
				//				error("Insufficient parameters given to the "+commandString+" command.",null);
				Props p = jerminal.getProps();
				Object o =  p.getProperty((String)operands.elementAt(0));
				return(o+"\n");
			}
		}

		try {
			Props p = jerminal.getProps();
			String name = (String)operands.elementAt(0);
			if(p.hasProperty(name)) {
				rval = p.getProperty(name);
			}
			//System.out.println("Setting "+name+" to "+operands.elementAt(1));
			p.setProperty(name,operands.elementAt(1));
		}
		catch(Exception e) {
			error("Exception in evaluate",e);
		}

		return(rval);
	}

}
