package com.cometway.jerminal.commands;

import com.cometway.jerminal.*;

import java.util.*;
import java.io.*;

/**
 *	Copyright (c) 2000-2001, Comet Way, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, 
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the 
 *     above copyright notice, this list of conditions 
 *     and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *   - Neither the name of Comet Way, Inc. nor the names of 
 *     its contributors may be used to endorse or promote 
 *     products derived from this software without specific 
 *     priorwritten permission. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIEDWARRANTIES OF MERCHANTABILITY AND FITNESS 
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.IN NO EVENT SHALL THE 
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; ORBUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 *  ON ANY THEORY OF LIABILITY, WHETHER INCONTRACT, STRICT LIABILITY, 
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 *  SUCH DAMAGE. 
 *
 * Revision history:
 */

/**
 * This command reads a file from the local file system and evaluates it line by line. The evaluated
 * command is printed out by the source command before it is evaluated.<p>
 * Synopsis:<p>
 * source [source file]<p>
 */
public class source extends UnixFileCommand
{
	public source(JerminalSession j) throws UnauthorizedAccessException
	{
		super(j);
		commandString = "source";
		helpString = "This command reads a file from the local file system and evaluates it line by line. The evaluated "+
			"command is printed out by the source command before it is evaluated.\n\n"+
			"Synopsis:\n"+
			"  source [source file]\n\n";

	}

	public Object evaluate(Vector operands)
	{
		Object rval = null;
		if(help(operands)) {
			return(helpString+"\n");
		}
		
		if(operands.size()==0) {
			rval = new String("Invalid argument");
		}
		else {
			File f = resolve((String)operands.elementAt(0));
			BufferedReader reader = null;
			try {
				reader = new BufferedReader(new FileReader(f));
				String line = reader.readLine();
				while(line!=null) {
					if(line.trim().length()>0) {
						print(line);
						jerminal.evaluate(line);
					}
					line = reader.readLine();
				}
			}
			catch(Exception e) {
				error("Could not source file "+operands.elementAt(0),e);
			}
			try {
				reader.close();
			}
			catch(Exception e) {;}
		}
		
		return(rval);
	}

}
